/*
JMP Clinical Analytical Process

Name: Adverse Events Narrative
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
This process will generate an AE narrative.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2014 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName PdfRtf InPath AInPath RVInData WhereStmt AEWhereStmt AESerOnly TEAEOnly OutName OutPath ServerOutPath MacroPath IncSF PopulationSet
        DSWhereStmt IncENRL ProcessName AENarrativeScript AENarrativeScriptName AENarrativeOuputFilePath AENarrativeOuputFilename AENarrative_OutData;
%global AeWindow CMedOption CMOption CmIndFl AeSev SaeList SaeTextList InvNam InvId CmIndc AeContrt SaePresList SaePresTextList MedHOption MHOption SubjectFilter TrtOffSet 
        FFlA FFDomA FFDomAU StOrA FRoundA FDecA FBaseA FTypeA FRangeA FCodeA FWhereStmtA FTestListA FWhereListA FBlFlagA FVarA FLabA FLabLenA
        FFlB FFDomB FFDomBU StOrB FRoundB FDecB FBaseB FTypeB FRangeB FCodeB FWhereStmtB FTestListB FWhereListB FBlFlagB FVarB FLabB FLabLenB
        FFlC FFDomC FFDomCU StOrC FRoundC FDecC FBaseC FTypeC FRangeC FCodeC FWhereStmtC FTestListC FWhereListC FBlFlagC FVarC FLabC FLabLenC
        HoFl HospOption HoWinYN HoWindow HoWhereStmt HOOption PVSign HoWin        
        EXFl EXDaily EXPVDose MergeSuppDom TOCFl EVTSPRFL runAsDrillDown NarrativeTemplate CMWindow SUBJIDFl EXENDTCFl AEDeltaDrop IgnoreTEAEFlags CTermOnly ExAEData MultWhere LimitRelEvents
        MedHForm ConMedTab RelRecCM TabStartNum TabPrefix
        FTransA FTransColA FDupsA FTransB FTransColB FDupsB FTransC FTransColC FDupsC
        ;

*ProcessBody;

%macro AENARR;

/*    
$CVS_START$
$ID: NarrDM$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.USUBJID and DM.AGE and DM.RACE and DM.SEX and (DM.INVNAM or DM.INVID or DM.SITEID) and DM.RFSTDTC$
$CMP: CPV.{"Use subject identifier for study in lieu of unique subject identifier, if available","Yes"}$ $UE: DM.SUBJID$ 
      $NOTES: SUBJID will be used instead of USUBJID in narrative text, if available$
$CMP: OSYS$ $UE: DM.ARM$ $NOTES: If ARM is missing, it will be created and filled as Blinded Treatment.$
$CMP: OSYS$ $UE: DM.ACTARM $ $NOTES: Actual arm reported if available.$
$CMP: OSYS$ $UE: DM.AGEU $ $NOTES: age units used if available.$
$CMP: OSYS$ $UE: DM.COUNTRY $ $NOTES: included in data set if available.$
$CMP: OSYS$ $UE: DM.STUDYID $ $NOTES: included in data set if available.$
$CMP: OSYS$ $UE: DM.SUBJID $ $NOTES: included in data set if available.$
$CVS_STOP$

$CVS_START$
$ID: NarrAE$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"TreatmentEmergentEvents","CalcStudyDay_Events"}}$
$INCD: {"AE"}$
$CMP: CPV.{"Include serious adverse events only","Yes"}$ $UE: AE.AESER$ $NOTES:  Only records with AESER = Y will be kept in analysis.$
$CMP: CPV.{"Limit reported related events to those that are serious","Yes"}$ $UE: AE.AESER$
$CMP: RSYS$ $UE: AE.USUBJID and AE.AESTDTC and AE.AEENDTC and AE.AETERM$
$CMP: OSYS$ $UE: AE.AEACN$ $NOTES: AE action if available.$ 
$CMP: OSYS$ $UE: AE.AEACNOTH$ $NOTES: Other AE action if available.$ 
$CMP: OSYS$ $UE: AE.AEDECOD$ $NOTES: AE coded term reported if available.$ 
$CMP: OSYS$ $UE: AE.AEOUT$ $NOTES: AE outcome reported if available.$
$CMP: OSYS$ $UE: AE.AECONTRT$ $NOTES:Concomitant or additional treatment given reported if available.$ 
$CMP: OSYS$ $UE: AE.AEREL$ $NOTES: AE relationship to study medication reported if available.$  
$CMP: OSYS$ $UE: AE.AETOXGR or AE.AESEV$ $NOTES: AE severity reported if available.$   
$CMP: OSYS$ $UE: AE.AESCAN or AE.AESCONG or AE.AESDISAB or AE.AESDTH or AE.AESHOSP or AE.AESLIFE or AE.AESOD or AE.AESMIE $ $NOTES: AE Reasons for seriousness reported if available.$
$CMP: OSYS$ $UE: AE.AESEQ$ $NOTES: Used for uniqueness of AE or generated if not available.$ 
$CVS_STOP$

$CVS_START$
$ID: NarrCM optional requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"InterventionOccur","CalcStudyDay_InterventionsO"}}$
$INCD: {"CM"}$
$CMP: CPV.{"Include a summary table of a concomitant medications","Yes"}$ $UE: CM.CMTRT and CM.CMDECOD and CM.CMINDC and (CM.CMDOSE or CM.CMDOSTXT) and CM.CMSTDTC and CM.CMENDTC$ $NOTES:If variables not present table is turned off.$
$CMP: CPV.{"Concomitant medications are reported in text using:","Reported name of drug"}$ $UE: CM.CMTRT$ $NOTES:  Used only if CM domain available.$
$CMP: CPV.{"Concomitant medications are reported in text using:","Standardized medication name"}$ $UE: CM.CMDECOD$ $NOTES:  Used only if CM domain available.$
$CMP: CPV.{"Use Related Records (RELREC) data set to identify concomitant medications that were given due to the event","Yes"}$ $UE: CM.CMSEQ and AE.AESEQ and AE.AECONTRT$ $NOTES: Used only when RELREC used.$
$CMP: CPV.{"Include concomitant medication indication in narrative text","Yes"}$ $UE: CM.CMINDC$ $NOTES:  Used only if CM domain available.$
$CMP: OSYS$ $UE: CM.USUBJID and CM.CMSTDTC and CM.CMENDTC and (CM.CMTRT or CM.CMDECOD)$ $NOTES: Minimal requirements to use CM, if available.$
$CVS_STOP$

$CVS_START$
$ID: NarrDS optional requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"CalcStudyDay_EventsO"}}$
$INCD: {"DS"}$
$CMP: OSYS$ $UE: DS.DSSTDTC and DS.DSDECOD and DS.DSTERM$  $NOTES: Minimal requirements to use DS, if available. Otherwise, DS not presented. $
$CMP: OSYS$ $UE: DS.DSCAT $  $NOTES: If available and no where statement provided, subset to DS records where = DISPOSITION EVENT.$
$CMP: OSYS$ $UE: DS.DSEPOCH $  $NOTES: If available and no where statement provided, subset to DS records with TREATMENT in value.$
$CVS_STOP$

$CVS_START$
$ID: NarrEX optional requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"CalcStudyDay_InterventionsO"}}$
$INCD: {"EX"}$
$CMP: CPV.{"Include exposure (EX) in the narrative","Yes"}$ $UE: EX.USUBJID$ $NOTES: EX domain must be available.$
$CMP: OSYS$ $UE: EX.EXDOSE and EX.EXDOSU and EX.EXTRT and EX.EXSTDTC and EX.EXENDTC$  
          $NOTES: Minimal requirements to use EX, if available. EX.EXENDTC can be imputed with EX.EXSTDTC using option$
$CVS_STOP$

$CVS_START$
$ID: NarrMH optional requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"EventOccur","CalcStudyDay_EventsO"}}$
$INCD: {"MH"}$
$CMP: CPV.{"Medical history terms are summarized as a:","Table"}$ $UE: MH.USUBJID and (MH.MHTERM or MH.MHDECOD) and MH.MHSTDTC $ $NOTES:  Used only if MH domain available.$
$CMP: CPV.{"Medical history terms are summarized as a:","List"}$ $UE: MH.USUBJID and (MH.MHTERM or MH.MHDECOD) and MH.MHSTDTC $ $NOTES:  Used only if MH domain available.$
$CMP: CPV.{"Medical history terms are reported using:","Reported term for the medical history"}$ $UE: MH.MHTERM$ $NOTES:  Used only if MH domain available.$
$CMP: CPV.{"Medical history terms are reported using:","Dictionary derived term"}$ $UE: MH.MHDECOD$ $NOTES:  Used only if MH domain available.$
$CMP: OSYS$ $UE: MH.USUBJID and (MH.MHTERM or MH.MHDECOD) and MH.MHSTDTC $  $NOTES: Minimal requirements to use MH, if available. Otherwise, MH not presented.$
$CVS_STOP$

$CVS_START$
$ID: NarrHO optional requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"CalcStudyDay_EventsO"}}$
$INCD: {"HO"}$
$CMP: CPV.{"Include healthcare encounters (HO) in the narrative","Yes"}$ $UE: HO.USUBJID$ $NOTES: HO domain must be available.$
$CMP: CPV.{"Healthcare encounters are reported using:","Reported term for the healthcare encounter"}$ $UE: HO.HOTERM$ $NOTES:  Used only if HO domain available and requested.$
$CMP: CPV.{"Healthcare encounters are reported using:","Dictionary derived term"}$ $UE: HO.HODECOD$ $NOTES:  Used only if HO domain available and requested.$
$CMP: OSYS$ $UE: HO.USUBJID and (HO.HOTERM or HO.HODECOD) and HO.HOSTDTC$  $NOTES: Minimal requirements to use HO, if available and requested. Otherwise, HO not presented.$
$CVS_STOP$

$CVS_START$
$ID: Non-LB optional findings requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"PreTreatmentFindingsO"}}$ $NOTES:Treatment emergence used to identify baseline and after dosing records$
$CMP: CPV.{"Use result or finding in:","Original Units"}$ $UE: FINDINGS.xxORRES $ 
$CMP: CPV.{"Use result or finding in:","Standard Units"}$ $UE: FINDINGS.xxSTRESN or FINDINGS.xxSTRESC$ 
$CMP: CPV.{"Refer to findings tests using:","Test code"}$ $UE: FINDINGS.xxTESTCD$ 
$CMP: CPV.{"Refer to findings tests using:","Test name"}$ $UE: FINDINGS.xxTEST$ 
$CMP: OSYS$ $UE: FINDINGS.USUBJID and FINDINGS.xxDTC and (FINDINGS.xxTESTCD or FINDINGS.xxTEST) and ((FINDINGS.xxSTRESN or FINDINGS.xxSTRESC) or FINDINGS.xxORRES)$ 
$NOTES: Minimal requirements to use FINDINGS.XX, if available and requested. Otherwise, FINDINGS.XX not presented.$
$CMP: OSYS$ $UE: FINDINGS.ABLFL or FINDINGS.xxBLFL$  $NOTES: Used for baseline if available.$
$CMP: OSYS$ $UE: FINDINGS.xxORRESU or FINDINGS.xxSTRESU$  $NOTES: Units used if available.$
$CVS_STOP$

$CVS_START$
$ID: LB optional findings requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"PreTreatmentFindingsO"}}$ $NOTES:Treatment emergence used to identify baseline and after dosing records$
$INCD: {"LB"}$
$CMP: CPV.{"Use result or finding in:","Original Units"}$ $UE: LB.LBORRES and LB.LBORNRLO and LB.LBORNRHI$ 
$CMP: CPV.{"Use result or finding in:","Standard Units"}$ $UE: LB.LBSTRESN and LB.LBSTNRLO and LB.LBSTNRHI$ 
$CMP: CPV.{"Refer to findings tests using:","Test code"}$ $UE: LB.LBTESTCD$ 
$CMP: CPV.{"Refer to findings tests using:","Test name"}$ $UE: LB.LBTEST$ 
$CMP: OSYS$ $UE: LB.USUBJID and LB.LBDTC and (LB.LBTESTCD or LB.LBTEST) and ((LB.LBSTRESN and LB.LBSTNRLO and LB.LBSTNRHI) or (LB.xxORRES and LB.LBORNRLO and LB.LBORNRHI)) and LB.LBNRIND$ 
$NOTES: Minimal requirements to use LB, if available and requested. Otherwise, LB not presented.$
$CMP: OSYS$ $UE: LB.ABLFL or LB.LBBLFL$  $NOTES: Used for baseline if available.$
$CMP: OSYS$ $UE: LB.LBORRESU or LB.LBSTRESU$  $NOTES: Units used if available.$
$CMP: OSYS$ $UE: LB.LBSTRESC or LB.LBSTNRC$  $NOTES: Character values and ref range used in cases where numeric is missing$
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = AENarrative; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./AENarrmacros.sas" / nosource;
%if &exiterror %then %goto exit;

%let includeds =;
%if %length(&AENarrative_OutData)>18 %then %let AENarrative_OutData = %substr(&AENarrative_OutData,1,18);
%if %length(&AENarrative_OutData) %then %let Outname = &AENarrative_OutData._;
%else %let Outname = &AENarrative_OutData;
%let AENarrative_OutData =;

************************************;
*** Translation Strings        *****;
************************************;

%let trReported = %nrbquote(%LocaleGet(Reported_Text));
%let trResults = %nrbquote(%LocaleGet(Results_Text));
%let trDictionary = %nrbquote(%LocaleGet(Dictionary_Text));
%let trStandardized = %nrbquote(%LocaleGet(Standardized_Text));
%let trMissing = %nrbquote(%LocaleGet(Missing_Text));
%let trLabel = %nrbquote(%LocaleGet(Label_Text));
%let trDispositionEvent = %nrbquote(%LocaleGet(DispositionEvent_Text));
%let trUncodedCM = %nrbquote(%LocaleGet(UncodedCM_Text));
%let trUncodedAE = %nrbquote(%LocaleGet(UncodedAE_Text));
%let trUncodedHO = %nrbquote(%LocaleGet(UncodedHO_Text));
%let trUncoded = %nrbquote(%LocaleGet(Uncoded_Text));
%let trGrade = %nrbquote(%LocaleGet(Grade_Text));
%let trStartUnknown = %nrbquote(%LocaleGet(Startdateunknown_Text));
%let trIndicationNotProv = %nrbquote(%LocaleGet(IndicationNotProv_Text));
%let trPlacebo = %nrbquote(%LocaleGet(Placebo_Text));
%let trVehicle = %nrbquote(%LocaleGet(Vehicle_Text));
%let trTreated = %nrbquote(%LocaleGet(Treated_Text));
%let trDispositionEvent = %nrbquote(%LocaleGet(DispositionEvent_Text));
%let trNoTreatment = %nrbquote(%LocaleGet(NoTreatment_Text));
%let trOnTreatment = %nrbquote(%LocaleGet(OnTreatment_Text));
%let trPreTreatment = %nrbquote(%LocaleGet(PreTreatment_Text));
%let trPostTreatment = %nrbquote(%LocaleGet(PostTreatment_Text));
%let trOf = %nrbquote(%LocaleGet(Of_Text));
%let trYesOne = %nrbquote(%LocaleGet(Y_Text));
%let trDay = %nrbquote(%LocaleGet(Day_Text));
%let trAsian = %nrbquote(%LocaleGet(Asian_Text));
%let trAfrican = %nrbquote(%LocaleGet(African_Text));
%let trAmerican = %nrbquote(%LocaleGet(American_Text));
%let trCaucasian = %nrbquote(%LocaleGet(Caucasian_Text));
%let trIndian = %nrbquote(%LocaleGet(Indian_Text));
%let trOriental = %nrbquote(%LocaleGet(Oriental_Text));
%let trAlaska = %nrbquote(%LocaleGet(Alaska_Text));
%let trNative = %nrbquote(%LocaleGet(Native_Text));
%let trHawaiian = %nrbquote(%LocaleGet(Hawaiian_Text));
%let trIslander = %nrbquote(%LocaleGet(Islander_Text));
%let trPacific = %nrbquote(%LocaleGet(Pacific_Text));
%let trNotRelated = %nrbquote(%LocaleGet(NotRelated_Text));
%let trRelated = %nrbquote(%LocaleGet(Related_Text));
%let trUnlikelyRelated = %nrbquote(%LocaleGet(UnlikelyRelated_Text));
%let trProbablyRelated = %nrbquote(%LocaleGet(ProbablyRelated_Text));
%let trPossiblyRelated = %nrbquote(%LocaleGet(PossiblyRelated_Text));
%let trDefinitelyRelated = %nrbquote(%LocaleGet(DefinitelyRelated_Text));
%let trUnlikely = %nrbquote(%LocaleGet(Unlikely_Text));
%let trNoOne = %nrbquote(%LocaleGet(N_Text));
%let trNot = %nrbquote(%LocaleGet(Not_Text));
%let trProbable = %nrbquote(%LocaleGet(Probable_Text));
%let trPossible = %nrbquote(%LocaleGet(Possible_Text));
%let trDefinite = %nrbquote(%LocaleGet(Definite_Text));
%let trDefinitely = %nrbquote(%LocaleGet(Definitely_Text));
%let trNone = %nrbquote(%LocaleGet(None_Text));
%let trProbably = %nrbquote(%LocaleGet(Probably_Text));
%let trPossibly = %nrbquote(%LocaleGet(Possibly_Text));
%let trRemote = %nrbquote(%LocaleGet(Remote_Text));
%let trRemotely = %nrbquote(%LocaleGet(Remotely_Text));
%let trRemotelyRelated = %nrbquote(%LocaleGet(RemotelyRelated_Text));
%let trYes = %nrbquote(%LocaleGet(Yes_Text));
%let trNo = %nrbquote(%LocaleGet(No_Text));
%let trYear = %nrbquote(%LocaleGet(year_Text));
%let trScreenFailure = %nrbquote(%LocaleGet(ScreenFailure_Text));
%let trNotTreated = %nrbquote(%LocaleGet(NotTreated_Text));
%let trNotAssigned = %nrbquote(%LocaleGet(NotAssigned_Text));
%let trNormal = %nrbquote(%LocaleGet(Normal_Text));
%let trHigh = %nrbquote(%LocaleGet(High_Text));
%let trLow = %nrbquote(%LocaleGet(Low_Text));
%let trRange = %nrbquote(%LocaleGet(Range_Text));
%let trNA = %nrbquote(%LocaleGet(NA_Text));
%let trBL = %nrbquote(%LocaleGet(BL_Text));
%let trAndText = %nrbquote(%LocaleGet(and_Text));
%let trUnknown = %nrbquote(%LocaleGet(Unknown_Text));
%let trRespectively = %nrbquote(%LocaleGet(respectively_Text));
%let trAtText = %nrbquote(%LocaleGet(at_Text));
%let trAENarrative = %nrbquote(%LocaleGet(AENarrative_Text));
%let trOpen = %nrbquote(%LocaleGet(Open_Text));
%let trEndDateTimeofTreatment = %nrbquote(%LocaleGet(EndDateTimeofTreatment_Text));
%let trCausalitywith = %nrbquote(%LocaleGet(Causalitywith_Text));
%let trCausalityof = %nrbquote(%LocaleGet(Causalityof_Text));
%let trActionTakenwith = %nrbquote(%LocaleGet(ActionTakenwith_Text));
%let trMultiple = %nrbquote(%LocaleGet(Multiple_Text));
%let trConMedTabNote = %nrbquote(%LocaleGet(ConMedTabNote_Text));
%let trRelRecCMNote = %nrbquote(%LocaleGet(RelRecCMNote_Text));
%let trVisit = %nrbquote(%LocaleGet(Visit_Text));

************************************;
*** Labels                     *****;
************************************;

%let FFDomAU = %unquote(&FFDomA);
%let FFDomBU = %unquote(&FFDomB);
%let FFDomCU = %unquote(&FFDomC);

%PARSEFINDLIST(code = A);
%PARSEFINDLIST(code = B);
%PARSEFINDLIST(code = C);

%if &FFDomAU = LB %then %do;
   %if %scan(%upcase(&FRangeA), 2, " ") = WITH %then %let FRangeA = ABNORMAL;
   %else %if %scan(%upcase(&FRangeA), 2, " ") = REGARDLESS %then %let FRangeA = ALL;
   %else %if %scan(%upcase(&FRangeA), 2, " ") = COMPARED %then %let FRangeA = DIFFERENT;
%end;
%if &FFDomBU = LB %then %do;
   %if %scan(%upcase(&FRangeB), 2, " ") = WITH %then %let FRangeB = ABNORMAL;
   %else %if %scan(%upcase(&FRangeB), 2, " ") = REGARDLESS %then %let FRangeB = ALL;
   %else %if %scan(%upcase(&FRangeB), 2, " ") = COMPARED %then %let FRangeB = DIFFERENT;
%end;
%if &FFDomCU = LB %then %do;
   %if %scan(%upcase(&FRangeC), 2, " ") = WITH %then %let FRangeC = ABNORMAL;
   %else %if %scan(%upcase(&FRangeC), 2, " ") = REGARDLESS %then %let FRangeC = ALL;
   %else %if %scan(%upcase(&FRangeC), 2, " ") = COMPARED %then %let FRangeC = DIFFERENT;
%end;

%let FLabA = ;
%let FLabB = ;
%let FLabC = ;
%let FLabLenA = 1;
%let FLabLenB = 1;
%let FLabLenC = 1;

%let i = 1;
%do %while(%length(%scan(%superq(findingsdomainslabel),&i,~)));
   %let templab = %sysfunc(lowcase(%scan(%superq(findingsdomainslabel),&i,~)));
   %if %index(%superq(templab), %nrbquote(%str(%())) %then %do;
      %if %index(%superq(templab), %nrbquote(%str(%())) > 1 %then %let templab = %substr(%superq(templab),1,%eval(%index(%superq(templab), %nrbquote(%str(%()))-1));
      %else %let templab = &tempdom;
      %if %length(%superq(templab)) %then %let templab = %sysfunc(trim(%superq(templab))); 
   %end; 
   %if %qsubstr(%superq(templab),1,3) = ecg %then %let templab = %sysfunc(tranwrd(%superq(templab), ecg, ECG));
   %let tempdom = %scan(&findingsdomains,&i,' ');
      
   %if &FFDomAU = &tempdom %then %do;
      %if &tempdom ne %nrbquote(%upcase(%superq(templab))) %then %let FLabA = %superq(templab);
      %else %let FLabA = %upcase(%superq(templab)) %sysfunc(lowcase(&trResults));
      %if %eval(%length(%superq(FLabA)) > &FLabLenA) %then %let FLabLenA = %length(%superq(FLabA));
   %end;
   %if &FFDomBU = &tempdom %then %do;
      %if &tempdom ne %nrbquote(%upcase(%superq(templab))) %then %let FLabB = %superq(templab);
      %else %let FLabB = %upcase(%superq(templab)) %sysfunc(lowcase(&trResults));
      %if %eval(%length(%superq(FLabB)) > &FLabLenB) %then %let FLabLenB = %length(%superq(FLabB));
   %end;
   %if &FFDomCU = &tempdom %then %do;
      %if &tempdom ne %nrbquote(%upcase(%superq(templab))) %then %let FLabC = %superq(templab);
      %else %let FLabC = %upcase(%superq(templab)) %sysfunc(lowcase(&trResults));
      %if %eval(%length(%superq(FLabC)) > &FLabLenC) %then %let FLabLenC = %length(%superq(FLabC));
   %end;
%let i = &i + 1;
%end;

************************************;
*** Check Requirements         *****;
************************************;

%if ^%length(&TabStartNum) %then %let TabStartNum = 1;

%if %scan(%upcase(&CMedOption),1," ") = %upcase(Reported) %then %let CMOption = CMTRT;
%else %if %scan(%upcase(&CMedOption),1," ") = %upcase(Standardized) %then %let CMOption = CMDECOD;

%let doCM = 0;
%if %sysfunc(indexw(&domains, CM, ' ')) %then %do;
   %if %sysfunc(findw(%upcase( &CMVARS), USUBJID)) and
       %sysfunc(findw(%upcase(&CMVARS), CMSTDTC)) and
       %sysfunc(findw(%upcase(&CMVARS), CMENDTC)) and
       %sysfunc(findw(%upcase(&CMVARS), %upcase(&CMOption))) %then %let doCM = 1;

   %if %sysfunc(findw(%upcase(&CMVARS), CMSEQ)) %then %let CMSEQ = CMSEQ;
   %else %let CMSEQ =;
%end;

%let RelRecCM = %substr(%upcase(&RelRecCM),1,1);

%if &RelRecCM = Y and &doCM %then %do;
   %if ^%length(&InPath) %then %do;
      %put ERROR: RELREC cannot be used for the current study. Please change dialog option to remove RELREC from consideration.;
      %put ERROR: No SDTM Path is defined for the current study.;
      %let exiterror = 1;
      %goto exit;
%end;

   %if %length(&relrecpresent) %then %do;   
      %if ^%sysfunc(findw(%upcase(&AEVARS), AECONTRT)) %then %do;
         %put ERROR: RELREC cannot be used for the current study. Please change dialog option to remove RELREC from consideration.;
         %put ERROR: AECONTRT is not available in the AE Domain.;
         %let exiterror = 1;
         %goto exit;      
      %end;

      %if ^%sysfunc(findw(%upcase(&AEVARS), AESEQ)) %then %do;
         %put ERROR: RELREC cannot be used for the current study. Please change dialog option to remove RELREC from consideration.;
         %put ERROR: AESEQ is not available in the AE Domain.;
         %let exiterror = 1;
         %goto exit;      
      %end;

      %if ^%sysfunc(findw(%upcase(&CMVARS), CMSEQ)) %then %do;
         %put ERROR: RELREC cannot be used for the current study. Please change dialog option to remove RELREC from consideration.;
         %put ERROR: CMSEQ is not available in the CM Domain.;
         %let exiterror = 1;
         %goto exit;      
      %end;   
   
      %if ^%sysfunc(exist(sdtm.relrec)) %then %do;
         %let inDompath=%sysfunc(pathname(sdtm));
         %let inDompath=&inDompath.\;
         %if %sysfunc(fileexist(&inDompath.relrec.xpt)) %then %do;
            %XPTtoSASFile(relrec.xpt,&inDompath);
            %let in_relrec= relrec;
         %end;
      %end;
      %else %let in_relrec = sdtm.relrec;
   
      %let _tid_ = %sysfunc(open(&in_relrec));
         %let usfl = %sysfunc(varnum(&_tid_,USUBJID));
         %let rdfl = %sysfunc(varnum(&_tid_,RDOMAIN));
         %let idfl = %sysfunc(varnum(&_tid_,IDVAR));
         %let refl = %sysfunc(varnum(&_tid_,RELID));
         %let vafl = %sysfunc(varnum(&_tid_,IDVARVAL));
      %let _tid_ = %sysfunc(close(&_tid_));

      %if not(&usfl and &rdfl and &idfl and &refl and &vafl) %then %do;
         %put ERROR: RELREC cannot be used for the current study. Please change dialog option to remove RELREC from consideration.;
         %put ERROR: At least one of the required variables USUBJID, DOMAIN, IDVAR, IDVARVAL, RELID are missing.;
         %let exiterror = 1;
         %goto exit;   
      %end;

      %let relrecidvar = ;         
      %GetLevels(&in_relrec,idvar,relrecidvar,_nLvls=nidvar,_vType=vtype);

      %if not(%sysfunc(findw(%upcase(&relrecidvar), CMSEQ)) and %sysfunc(findw(%upcase(&relrecidvar), AESEQ))) %then %do;
         %put ERROR: RELREC cannot be used for the current study. Please change dialog option to remove RELREC from consideration.;
         %put ERROR: IDVAR does not contain values for AESEQ and CMSEQ.;
         %let exiterror = 1;
         %goto exit;                    
      %end;         
   %end;
   %else %let RelRecCM = N;
%end;
%else %let RelRecCM = N;

%if %scan(%upcase(&MedHOption),1," ") = %upcase(Reported) %then %let MHOption = MHTERM;
%else %if %scan(%upcase(&MedHOption),1," ") = %upcase(Dictionary) %then %let MHOption = MHDECOD;

%let PdfRtf = rtf;

%if &EXDaily ne No and &EXDaily ne Yes %then %let EXDaily = No;
%if &EXPVDose ne No and &EXPVDose ne Yes %then %let EXPVDose = No;
%let CmIndFl = %substr(%upcase(&CmIndFl),1,1);
%let EXDaily = %substr(%upcase(&EXDaily),1,1);
%let EXPVDose = %substr(%upcase(&EXPVDose),1,1);
%let HoFl = %substr(%upcase(&HoFl),1,1);
%let ExFl = %substr(%upcase(&ExFl),1,1);
%let TOCFl = %substr(%upcase(&TOCFl),1,1);
%let CTermOnly = %substr(%upcase(&CTermOnly),1,1);

%let ConMedTabErr = 0;
%let ConMedTab = %substr(%upcase(&ConMedTab),1,1);
%if ^&doCM %then %let ConMedTab = N;
%if &doCM and &ConMedTab = Y and not (
   %sysfunc(findw(%upcase( &CMVARS), CMTRT)) and
   %sysfunc(findw(%upcase( &CMVARS), CMDECOD)) and
   %sysfunc(findw(%upcase( &CMVARS), CMINDC)) and
   (%sysfunc(findw(%upcase( &CMVARS), CMDOSE)) or %sysfunc(findw(%upcase( &CMVARS), CMDOSTXT))) and
   %sysfunc(findw(%upcase( &CMVARS), CMSTDTC)) and
   %sysfunc(findw(%upcase( &CMVARS), CMENDTC)))
%then %do;
   %let ConMedTab = N;
   %let ConMedTabErr = 1;
%end;

%if &EXPVDose = Y %then %let PVsign = %bquote(>); 
%else %if &EXPVDose = N %then %let PVsign = %bquote(>=); 

%let SaeList = AESCAN AESCONG AESDISAB AESDTH AESHOSP AESLIFE AESOD AESMIE;

%if ^%sysfunc(indexw(&domains, AE, ' ')) %then %do;
   %put [MS_S_0028];
   %put ERROR: The AE domain is not available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if not (%sysfunc(findw(%upcase( &AEVARS), USUBJID)) and
    %sysfunc(findw(%upcase(&AEVARS), AESTDTC)) and
    %sysfunc(findw(%upcase(&AEVARS), AEENDTC)) and
    %sysfunc(findw(%upcase(&AEVARS), AETERM))) %then %do;
      %put [MS_S_0029];
   %put ERROR: The requirements for the AE domain are not present.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(indexw(&domains, DM, ' ')) %then %do;
   %put [MS_S_0030];
   %put ERROR: The DM domain is not available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^&DMBlind %then %do;
%if not (%sysfunc(findw(%upcase( &DMVARS), USUBJID)) and
    %sysfunc(findw(%upcase(&DMVARS), AGE)) and
    %sysfunc(findw(%upcase(&DMVARS), ARM)) and
    %sysfunc(findw(%upcase(&DMVARS), RACE)) and
    %sysfunc(findw(%upcase(&DMVARS), SEX)) and
    %sysfunc(findw(%upcase(&DMVARS), RFSTDTC)) and
   (%sysfunc(findw(%upcase(&DMVARS), INVNAME)) or
    %sysfunc(findw(%upcase(&DMVARS), INVID)) or
    %sysfunc(findw(%upcase(&DMVARS), SITEID)))) %then %do;
      %put [MS_S_0031];
   %put ERROR: The requirements for the DM domain are not present.;
   %let exiterror = 1;
   %goto exit;
%end;
%end;
%else %do;
   %if not (%sysfunc(findw(%upcase( &DMVARS), USUBJID)) and
       %sysfunc(findw(%upcase(&DMVARS), AGE)) and
       %sysfunc(findw(%upcase(&DMVARS), RACE)) and
       %sysfunc(findw(%upcase(&DMVARS), SEX)) and
       %sysfunc(findw(%upcase(&DMVARS), RFSTDTC)) and
      (%sysfunc(findw(%upcase(&DMVARS), INVNAME)) or
       %sysfunc(findw(%upcase(&DMVARS), INVID)) or
       %sysfunc(findw(%upcase(&DMVARS), SITEID)))) %then %do;
         %put [MS_S_0031];
      %put ERROR: The requirements for the DM domain are not present.;
      %let exiterror = 1;
      %goto exit;
   %end;
%end;

%let doDS = 0;
%if %sysfunc(indexw(&domains, DS, ' ')) %then %do;
   %if %sysfunc(findw(%upcase( &DSVARS), USUBJID)) and
       %sysfunc(findw(%upcase(&DSVARS), DSSTDTC)) and
       %sysfunc(findw(%upcase(&DSVARS), DSDECOD)) and
       %sysfunc(findw(%upcase(&DSVARS), DSTERM)) %then %let doDS = 1;
%end;

%if &ExFl = Y and ^%sysfunc(indexw(&domains, EX, ' ')) %then %do;
   %put [MS_S_0032];
   %put ERROR: The EX domain is not available for the current study. Deselect the option to include Exposure (EX);
   %let exiterror = 1;
   %goto exit;
%end;

%let doEX = 0;
%if %sysfunc(indexw(&domains, EX, ' ')) %then %do;
   %if &ExFl = Y and
       %sysfunc(findw(%upcase(&EXVARS), USUBJID)) and
       %sysfunc(findw(%upcase(&EXVARS), EXDOSE)) and
       %sysfunc(findw(%upcase(&EXVARS), EXDOSU)) and
       %sysfunc(findw(%upcase(&EXVARS), EXSTDTC)) and
       %sysfunc(findw(%upcase(&EXVARS), EXTRT)) %then %let doEX = 1;
%end;

%if &doEx %then %do;
   %if ^%sysfunc(findw(%upcase(&EXVARS), EXENDTC)) and %upcase(&EXENDTCFl) = NO %then %do;
      %put ERROR: Treatment end date not detected (EXENDTC). Confirm that treatment end date is equivalent to the start date (EXSTDTC) by selecting the dialog option;
      %let exiterror = 1;
      %goto exit;
   %end;  
   
   *** if dosing end date detected ignore the option ***;
   %if %sysfunc(findw(%upcase(&EXVARS), EXENDTC)) and %upcase(&EXENDTCFl) = YES %then %let EXENDTCFl = No;
%end;

%let doMH = 0;
%if %sysfunc(indexw(&domains, MH, ' ')) %then %do;
   %if %sysfunc(findw(%upcase( &MHVARS), USUBJID)) and
       %sysfunc(findw(%upcase(&MHVARS), MHSTDTC)) and
      (%sysfunc(findw(%upcase(&MHVARS), MHTERM)) or
       %sysfunc(findw(%upcase(&MHVARS), MHDECOD))) %then %let doMH = 1;
%end;

%if &HoFl = Y and ^%sysfunc(indexw(&domains, HO, ' ')) %then %do;
   %put [MS_S_0033];
   %put ERROR: The HO domain is not available for the current study. Deselect the option to include Healthcare Encounters (HO);
   %let exiterror = 1;
   %goto exit;
%end;

%let doHO = 0;
%if %sysfunc(indexw(&domains, HO, ' ')) %then %do;
   %if &HoFl = Y and
       %sysfunc(findw(%upcase( &HOVARS), USUBJID)) and
       %sysfunc(findw(%upcase(&HOVARS), HOSTDTC)) and
      (%sysfunc(findw(%upcase(&HOVARS), HOTERM)) or
       %sysfunc(findw(%upcase(&HOVARS), HODECOD))) %then %let doHO = 1;
%end;

%if %upcase(&LimitRelEvents) = YES and ^%sysfunc(findw(%upcase(&AEVARS), AESER)) %then %do;
   %put ERROR: Cannot limit reported related events to those that are serious since serious event flag (AESER) is not available;
   %let exiterror = 1;
   %goto exit;
%end;

%if %upcase(&FFlA) = YES %then %do;
   %FINDINIT(code = A);
   %if &exiterror %then %goto exit;
%end;

%if %upcase(&FFlB) = YES %then %do;
   %FINDINIT(code = B);
   %if &exiterror %then %goto exit;
%end;

%if %upcase(&FFlC) = YES %then %do;
   %FINDINIT(code = C);
   %if &exiterror %then %goto exit;
%end;

%AESerOnlyToAEWhereStmt;
%WHEREDELTA(deltastate = AEDeltaDrop, wherestate = AEWhereStmt, deltaflag = AERFlg);

************************************;
*** Check for Presence of Vars *****;
************************************;

%let HOWin =;
%if &doHO %then %do;
   %if %scan(%upcase(&HospOption),1," ") = %upcase(&trReported) %then %let HOOption = HOTERM;
   %else %if %scan(%upcase(&HospOption),1," ") = %upcase(&trDictionary) %then %let HOOption = HODECOD;
   
   %if &HOOption = HODECOD and ^%sysfunc(findw(%upcase(&HOVARS), HODECOD)) %then %do;
         %put [MS_S_0034];
      %put ERROR: The &trDictionary term for healthcare encounters is not available.  Please change option;
      %let exiterror = 1;
      %goto exit;
   %end;    

   %if %sysfunc(findw(%upcase(&HOVARS), HODECOD)) %then %let HODECOD = HODECOD;
   %else %let HODECOD = ;  

   %if %sysfunc(findw(%upcase(&HOVARS), HOSTDY)) %then %let HOSTDY = HOSTDY;
   %else %let HOSTDY = ;   

   %if %scan(%upcase(&HoWinYN),1," ") = ALL %then %let HOWin = ALL;
   %else %let HOWin = WIN;
%end;

%if ^%sysfunc(findw(%upcase(&DMVARS), RFSTDTC)) %then %do;
   %put [MS_S_0035];
   %put ERROR: RFSTDTC is a required variable from the DM domain.;
   %let exiterror=1;
   %goto exit;
%end;

%let CMINDC =;
%let CMDECOD =;
%if &doCM %then %do;
   %if %sysfunc(findw(%upcase(&CMVARS), CMINDC)) %then %let CMINDC = CMINDC;
   %else %let CMINDC =;
   %if %sysfunc(findw(%upcase(&CMVARS), CMDECOD)) %then %let CMDECOD = CMDECOD;
   %else %let CMDECOD =;

   %if &CMOption = CMDECOD and ^%sysfunc(findw(%upcase(&CMVARS), CMDECOD)) %then %do;
         %put [MS_S_0036];
      %put ERROR: The &trStandardized term for concomitant medications is not available.  Please change option;
      %let exiterror = 1;
      %goto exit;
   %end;   
%end;

%if &doMH %then %do;
   %if &MHOption = MHDECOD and ^%sysfunc(findw(%upcase(&MHVARS), MHDECOD)) %then %do;
         %put [MS_S_0037];
      %put ERROR: The &trDictionary term for medical history is not available.  Please change option;
      %let exiterror = 1;
      %goto exit;
   %end;   
   %if &MHOption = MHTERM and ^%sysfunc(findw(%upcase(&MHVARS), MHTERM)) %then %let MHOption = ;

   %if %sysfunc(findw(%upcase(&MHVARS), MHSTDTC)) %then %let MHSTDTC = MHSTDTC;
   %else %let MHSTDTC = ;

   %if %sysfunc(findw(%upcase(&MHVARS), MHDECOD)) %then %let MHDECOD = MHDECOD;
   %else %let MHDECOD = ;  
%end;

%if &doDS %then %do;
   %if %sysfunc(findw(%upcase(&DSVARS), DSSTDTC)) %then %let DSSTDTC = DSSTDTC;
   %else %let DSSTDTC =;

   %if %sysfunc(findw(%upcase(&DSVARS), DSCAT)) %then %let DSCAT = DSCAT;
   %else %let DSCAT =;

   %if %sysfunc(findw(%upcase(&DSVARS), DSEPOCH)) %then %let DSEPOCH = DSEPOCH;
   %else %let DSEPOCH =;
%end;

%if %sysfunc(findw(%upcase(&DMVARS), INVNAM)) %then %let INVNAM = INVNAM;
%else %let INVNAM =;
%if %sysfunc(findw(%upcase(&DMVARS), INVID)) %then %let INVID = INVID;
%else %let INVID =;
%if %sysfunc(findw(%upcase(&DMVARS), STUDYID)) %then %let STUDYID = STUDYID;
%else %let STUDYID =;
%if %sysfunc(findw(%upcase(&DMVARS), COUNTRY)) %then %let COUNTRY = COUNTRY;
%else %let COUNTRY =;
%if %sysfunc(findw(%upcase(&DMVARS), SUBJID)) %then %let SUBJID = SUBJID;
%else %let SUBJID =;

%if %sysfunc(findw(%upcase(&AEVARS), AEDECOD)) %then %let AEDECOD = AEDECOD;
%else %let AEDECOD =;
%if %sysfunc(findw(%upcase(&AEVARS), AECONTRT)) %then %let AECONTRT = AECONTRT;
%else %let AECONTRT =;
%if %sysfunc(findw(%upcase(&AEVARS), AESER)) %then %let AESER = AESER;
%else %let AESER =;
%if %sysfunc(findw(%upcase(&AEVARS), AEREL)) %then %let AEREL = AEREL;
%else %let AEREL =;
%if %sysfunc(findw(%upcase(&AEVARS), AEACN)) %then %let AEACN = AEACN;
%else %let AEACN =;
%if %sysfunc(findw(%upcase(&AEVARS), AEACNOTH)) %then %let AEACNOTH = AEACNOTH;
%else %let AEACNOTH =;
%if %sysfunc(findw(%upcase(&AEVARS), AEOUT)) %then %let AEOUT = AEOUT;
%else %let AEOUT =;

%if %sysfunc(findw(%upcase(&AEVARS), AETOXGR)) %then %let AESEV = AETOXGR;
%else %if %sysfunc(findw(%upcase(&AEVARS), AESEV)) %then %let AESEV = AESEV;
%else %let AESEV = ;

%if %length(&AESEV) %then %let AESEV2 = &AESEV.2;
%else %let AESEV2 = ;

%let AEDropArm = ;
%if %sysfunc(findw(%upcase(&AEVARS), ARM)) %then %let AEDropARM = ARM;
%if %sysfunc(findw(%upcase(&AEVARS), ACTARM)) %then %let AEDropARM = &AEDropARM ACTARM;

%if %sysfunc(findw(%upcase(&DMVARS), ACTARM)) %then %let ACTARM = ACTARM;
%else %let ACTARM=;
%if %sysfunc(findw(%upcase(&DMVARS), AGEU)) %then %let AGEU = AGEU;
%else %let AGEU=;

%if ^%sysfunc(exist(&in_ae)) %then %do;
   %XPTtoSASFile2(&in_ae);
   %let in_ae=ae;
%end;

%let _ind_ = %sysfunc(open(&in_ae));
%let SaePresList = ;
%let SaePresTextList = ;
%let SaePresFlag = 0;
%do i = 1 %to 8;
   %let temp = %scan(&SaeList, &i, " ");
   %let vn = %sysfunc(varnum(&_ind_,&temp));
   %if &vn %then %do;
      %let templabel = %nrbquote(%sysfunc(varlabel(&_ind_,&vn)));
      %if ^%length(&templabel) %then %let templabel = &trMissing &temp &trLabel;
      %let SaePresList = &SaePresList &temp;
      %if ^&SaePresFlag %then %let SaePresTextList = &templabel;
      %else %if &SaePresFlag %then %let SaePresTextList = &SaePresTextList~&templabel;
      %if ^&SaePresFlag %then %let SaePresFlag = 1;
   %end;
%end;   
%let _ind_ = %sysfunc(close(&_ind_));

************************************;
******* Data ***********************;
************************************;

*** Prepare ADSL ***;
%PREPADSL;
%if &exiterror %then %goto exit;

*** External ***;

%if %length(&ExAEData) %then %do;  
   %GetOrigFileName(ExAEData,OrigExData);
   %PathName(&ExAEData);
   %if &exiterror %then %do;
      %put [MS_S_0163];
      %put ERROR: Input dataset specification is invalid.;
      %goto gexit;
   %end;
   %let ExAEPath = &TmpPath;
   libname ExAELib "&ExAEPath";
   %let ExAEData = &TmpName;
%end;

*** the selected subjects ***;
data adsltmp;
   set &in_adsl;
   %DATE9(var=rfstdtc,return=rfstdt);   
run;

*** all events for selected subjects ***;
%LOADDATA(domlist = AE,mergesupp = 0,checkreview=1,_NarrFlag = 0);

%if %sysfunc(findw(%upcase(&AEVARS), AESEQ)) %then %do;
   proc sort data = ae out = ae(where = (aeterm ne ""));
      by usubjid aeseq;
   run;
%end;
%else %do;
   proc sort data = ae out = ae(where = (aeterm ne ""));
      by usubjid aestdtc aeterm;
   run;

   data ae;
      set ae;
      by usubjid;
      retain aeseq;
      if first.usubjid then aeseq = 1;
      else aeseq = aeseq + 1;
   run;
%end;

*** merge in covariates ***;
data ae;
   merge adsltmp(in = ina) ae(in = inb %if %length(&AEDropArm) %then %do; drop = &AEDropArm %end;);
   by usubjid;      
   if ina and inb;
   if d9partial_RFSTDTC then aeterm = cats(AETERM,"*");
run;
%NObsVars(ae);
%if ^&nobs %then %do;
   %put [MS_S_0038];
   %put ERROR: Selected subjects do not have any Adverse Events.;
   %let exiterror = 1;
   %goto exit;
%end;

data ae;
   %if ^%length(&AEDECOD) %then %do;
      length aedecod $ 30;
   %end;
   set ae;
   if aedecod = "" then aedecod = "&trUncodedAE";
   *** formatted dates for narrative ***;
   %DATE9(var=aestdtc,return=aestdt);
   %DATE9(var=aeendtc,return=aeendt);
   
   *** study days: use what is provided or calculate ***;
   %calculateStudyDays(AE);
   drop aestdy aeendy log2_aedur %if ^%sysfunc(findw(%upcase(&AEVARS), AEDUR)) %then aedur; errdt:;
   if aeendt=. then endy = .;
   rename endy = AEENDY stdy = AESTDY;
   
   %if &AeSev = AESEV %then %do;
      aeterm2 = catx(" ", lowcase(aedecod), cats("(",lowcase(&AeSev),")"));
   %end;
   %else %if &AeSev = AETOXGR %then %do;
      aeterm2 = catx(" ", lowcase(aedecod), cats("(",catx(" ", "&trGrade", lowcase(&AeSev)),")"));
   %end;
   %else %do;
      aeterm2 = lowcase(aedecod);
   %end;
   
   if AESTDTC ne "" then do;
      if partial_AESTDTC then aestdtcp = cats(put(aestdt,&IntDateFormat..),"*");
      else aestdtcp = put(aestdt,&IntDateFormat..);
   end;
   if AEENDTC ne "" then do;
      if partial_AEENDTC then aeendtcp = cats(put(aeendt,&IntDateFormat..),"*");
      else aeendtcp = put(aeendt,&IntDateFormat..);   
   end;
run;

%let AENarrative_OutData_event =;
%PrepOutDS(AENarrative_OutData_event,OutDataName_event, &Outname.ae);
%let includeds = &includeds &OutDataName_event..sas7bdat,;

*** all events for subjects of interest ***;
data &AENarrative_OutData_event;
   set ae(keep = &AEVARS AESTDY AEENDY partial_AESTDTC partial_AEENDTC);
   if partial_AESTDTC = 1 then AESTDY = .;
   if partial_AEENDTC = 1 then AEENDY = .;
   drop partial_AESTDTC partial_AEENDTC;
run;

%let in_ae = ae;

%include "&MacroPath./TEAEOTFU.sas";
%if &exiterror %then %goto exit; 

*** reduce vars to those needed for nearby events ***;
data ae(keep = usubjid aeseq aestdt aedecod &AeSev aeterm2 aestdtcp);
   set ae;
   %if %upcase(&LimitRelEvents) = YES %then %do; 
      where (AESER="Y" or AESER="&trY");
   %end;
   *** flag for partial date for nearby ae ***;
   if partial_AESTDTC then do;
      aedecod = cats(aedecod,"*");
      aeterm2 = cats(aeterm2,"*");
   end;
run;

*** selected events for selected subjects ***;
data aework;
   set &in_ae;
   %if %length(&AEWhereStmt) %then %do;
      where %unquote(&AEWhereStmt);
   %end;
run;

*** determine presence of MULTIPLE flags ***;
%if %length(&AEREL) %then %do;
   proc sql noprint;         
      select distinct upcase(aerel)
         into :aerelvals separated by '~'
         from aework;
   quit;
   %if %sysfunc(findw(&aerelvals,%upcase(&trMultiple), ~)) %then %let AERELMULT = 1;
   %else %let AERELMULT = 0;
%end;
%else %let AERELMULT = 0;
%if %length(&AEACN) %then %do;
   proc sql noprint;         
      select distinct upcase(aeacn)
         into :aeacnvals separated by '~'
         from aework;
   quit;
   %if %sysfunc(findw(&aeacnvals,%upcase(&trMultiple), ~)) %then %let AEACNMULT = 1;
   %else %let AEACNMULT = 0;
%end;
%else %let AEACNMULT = 0;

%if ^%sysfunc(findw(&suppdomains, %upcase(SUPPAE))) %then %do;
   %let AEACNMULT = 0;
   %let AERELMULT = 0;
%end;

%if &syserr > 4 %then %do;
   %put [MS_S_0039];
   %put ERROR: The specified filter &AEWhereStmt is invalid.  Change the filter to be valid for the domain you are considering.;
   %let exiterror = 1;
   %goto exit;
%end;
%NObsVars(aework);
%if ^&nobs %then %do;
   %put [MS_S_0040];
   %put ERROR: No observations satisfy &AEWhereStmt or other event filters.;
   %put Please relax criteria.;
   %let exiterror = 1;
   %goto exit;
%end;

*** subjects with selected AEs available ***;
data aesubs(keep = usubjid rfstdtc);
   set aework;
   by usubjid;
   if first.usubjid;
run;

%if &AEACNMULT or &AERELMULT %then %do;

	%if ^%sysfunc(exist(sdtm.SUPPAE)) %then %do;
		%let inDompath=%sysfunc(pathname(sdtm));
		%let inDompath=&inDompath.\;
		%if %sysfunc(fileexist(&inDompath.suppae.xpt)) %then %do;
			%XPTtoSASFile(suppae.xpt,&inDompath);
			%let in_suppae= suppae;
		%end;
	%end;
	%else %let in_suppae = sdtm.suppae;
	
   data aemult;
     if 0 then set aesubs;
     if _n_ = 1 then do;
        declare hash h(dataset:"aesubs");
        h.defineKey("USUBJID");
        h.defineData(ALL:"YES");
        h.defineDone();
     end;
     set &in_suppae(keep = USUBJID IDVARVAL QNAM QLABEL QVAL where = (substr(QNAM,1,3)="REL" or substr(QNAM,1,5)="AEREL" or substr(QNAM,1,3)="ACN" or substr(QNAM,1,5)="AEACN"));
     if h.find() = 0;
   run;

   proc sort data = aemult out = aemult;
     by USUBJID IDVARVAL;
   run;

   data aemult(keep = usubjid aeseq %if &AERELMULT %then %do; aerelmultterm2 %end; %if &AEACNMULT %then %do; aeacnmultterm2 %end;);
      length %if &AERELMULT %then %do; aerelmultterm2 %end; %if &AEACNMULT %then %do; aeacnmultterm2 %end; $ 200;
      set aemult;
      by usubjid IDVARVAL;
      %if %length(&MultWhere) %then %do;
         where %unquote(&MultWhere);
      %end;      
      retain %if &AERELMULT %then %do; aerelmultterm2 %end; %if &AEACNMULT %then %do; aeacnmultterm2 %end;;
      aeseq = idvarval + 0;
      %if &AERELMULT %then %do;
         if substr(QNAM,1,3)="REL" or substr(QNAM,1,5)="AEREL" then do;
            if index(upcase(QLABEL), upcase("&trCausalitywith")) then treatment = strip(substr(QLABEL,index(upcase(QLABEL), upcase("&trCausalitywith"))+length("&trCausalitywith")));
           else if index(upcase(QLABEL), upcase("&trCausalityof")) then treatment = strip(substr(QLABEL,index(upcase(QLABEL), upcase("&trCausalityof"))+length("&trCausalityof")));
         end;
      %end;
      %if &AEACNMULT %then %do;
         if substr(QNAM,1,3)="ACN" or substr(QNAM,1,5)="AEACN" then do;    
            if index(upcase(QLABEL), upcase("&trActionTakenwith")) then treatment = strip(substr(QLABEL,index(upcase(QLABEL), upcase("&trActionTakenwith"))+length("&trActionTakenwith")));
         end;
      %end;
      %if &AERELMULT %then %do;
         if substr(QNAM,1,3)="REL" or substr(QNAM,1,5)="AEREL" then do;
               aerelmultterm = catx(" ", lowcase(qval), "to", treatment);
         end;
      %end;
      %if &AEACNMULT %then %do;
         if substr(QNAM,1,3)="ACN" or substr(QNAM,1,5)="AEACN" then do;   
               aeacnmultterm = catx(" ", lowcase(qval), "for", treatment);
         end;
      %end;
      %if &AERELMULT %then %do;
         if first.IDVARVAL then aerelmultterm2 = aerelmultterm;
         else aerelmultterm2 = catx("~", aerelmultterm2, aerelmultterm);
      %end;
      %if &AEACNMULT %then %do;
         if first.IDVARVAL then aeacnmultterm2 = aeacnmultterm;
         else aeacnmultterm2 = catx("~", aeacnmultterm2, aeacnmultterm);
      %end;
      if last.IDVARVAL;
   run;
   
   proc sort data = aemult out = aemult;
     by usubjid aeseq;
   run;
%end;

*** all events for subjects of interest with events of interest ***;
data &AENarrative_OutData_event;
   merge aesubs(keep = usubjid in = ina) &AENarrative_OutData_event;
   by usubjid;
   if ina;
run;

*** immediately subset to relevant MH records ***;
%if &doMH %then %do;
   %LOADDATA(domlist = MH,mergesupp=0, checkreview=0, _TempWhereStmt = %str(mhterm ne ""), _TempKeep = usubjid mhterm &MHDECOD &MHSTDTC, _NarrFlag = 1);   
%end;

*** immediately subset to relevant EX records ***;
%if &doEX %then %do;
   *** dropping study days since exendtc may be set to exstdtc requiring recalculation ***;
   %LOADDATA(domlist = EX,mergesupp=0, checkreview=0, _TempDrop = endy stdy, _Sortby = USUBJID EXSTDTC, _NarrFlag = 1);   
%end;

*** immediately subset to relevant CM records ***;
%if &doCM %then %do;
   %LOADDATA(domlist = CM,mergesupp=0, checkreview=0, _TempWhereStmt = %str(cmtrt ne ""), _Sortby = USUBJID CMSTDTC, _NarrFlag = 1);

   %let AENarrative_OutData_CM =;
   %PrepOutDS(AENarrative_OutData_CM,OutDataName_CM, &Outname.cm);
   %let includeds = &includeds &OutDataName_CM..sas7bdat,;

   data &AENarrative_OutData_CM;
      merge aesubs(keep = usubjid in = ina) cm(keep = &CMVARS %if %sysfunc(findw(%upcase(&CMVARS), CMSTDTC)) %then %do; partial_CMSTDTC STDY %end; 
                                               %if %sysfunc(findw(%upcase(&CMVARS), CMENDTC)) %then %do; partial_CMENDTC ENDY %end;);
      by usubjid;
      if ina;
      %if %sysfunc(findw(%upcase(&CMVARS), CMSTDTC)) %then %do;
         %if %sysfunc(findw(%upcase(&CMVARS), CMSTDY)) %then %do; 
            drop cmstdy;
         %end;             
         rename stdy = CMSTDY;
         if partial_CMSTDTC = 1 then CMSTDY = .;
         drop partial_CMSTDTC;
      %end;
      %if %sysfunc(findw(%upcase(&CMVARS), CMENDTC)) %then %do;
         %if %sysfunc(findw(%upcase(&CMVARS), CMENDY)) %then %do; 
            drop cmendy;
         %end;
         rename endy = CMENDY;
         if partial_CMENDTC = 1 then CMENDY = .;
         drop partial_CMENDTC;
      %end;         
   run;  

   proc sql;
      create table totalcmcnt as
         select distinct usubjid, count(usubjid) as totalcmcnt
         from cm
         group by usubjid
         order by usubjid;
   quit;
%end;

*** immediately subset to relevant DS records ***;
%if &doDS %then %do;
   %LOADDATA(domlist = DS,mergesupp=0, checkreview=0, _TempDrop = dsstdy dsendy log2_dsdur dsdur errdt:, _Sortby = USUBJID &DSSTDTC, _NarrFlag = 1);
%end;

%if &doHO %then %do;
   %LOADDATA(domlist = HO,mergesupp=0, checkreview=0, _TempWhereStmt = %bquote(&HOWhereStmt), _TempKeep = usubjid hoterm &HODECOD hostdtc stdy, _Sortby = USUBJID HOSTDTC, _NarrFlag = 1);    

   %NObsVars(ho);
   %if ^&nobs %then %do;
      %put [MS_S_0041];
      %put ERROR: No observations satisfy &HOWhereStmt. Please relax criteria.;
      %let exiterror = 1;
      %goto exit;
   %end;
%end;

%if %upcase(&FFlA) = YES %then %do;
   %LOADDATA(domlist = &FFDomAU,mergesupp=0, checkreview=0, _TempWhereStmt = %bquote(&FWhereStmtA), _NarrFlag = 1);    
   
   %NARRBYLIST(domain = &FFDomAU, code = A);

   proc datasets nolist;
      change &FFDomAU = domA;
   quit;

   data domA;
      set domA;
      %if %scan(%upcase(&StOrA), 1, " ") = ORIGINAL %then %do;
         if missing(&FFDomAU.ORRES) then delete;
      %end;
      %else %do;
         %if %sysfunc(findw(%upcase( &&&FFDomAU.VARS), &FFDomAU.STRESC)) and %sysfunc(findw(%upcase( &&&FFDomAU.VARS), &FFDomAU.STRESN)) %then %do;
            if missing(&FFDomAU.STRESC) and missing(&FFDomAU.STRESN) then delete;
         %end;
         %else %if %sysfunc(findw(%upcase( &&&FFDomAU.VARS), &FFDomAU.STRESC)) and ^%sysfunc(findw(%upcase( &&&FFDomAU.VARS), &FFDomAU.STRESN)) %then %do;
            if missing(&FFDomAU.STRESC) then delete;
         %end;
         %else %if ^%sysfunc(findw(%upcase( &&&FFDomAU.VARS), &FFDomAU.STRESC)) and %sysfunc(findw(%upcase( &&&FFDomAU.VARS), &FFDomAU.STRESN)) %then %do;
            if missing(&FFDomAU.STRESN) then delete;
         %end;       
      %end;
      %if &FFDomAU = LB %then %do;
         if missing(LBNRIND) then delete;
      %end;
   run;
   %NObsVars(domA);
   %if ^&nobs %then %do;
      %put [MS_S_0042];
      %put ERROR: No observations satisfy &FWhereStmtA. Please relax criteria.;
      %let exiterror = 1;
      %goto exit;
   %end;
         
   %let AENarrative_OutData_domA =;
   %PrepOutDS(AENarrative_OutData_domA,OutDataName_domA, &Outname.&FFDomAU);
   %let includeds = &includeds &OutDataName_domA..sas7bdat,;

   *** all findings for subjects of interest with events of interest ***;
   %if ^%length(&FTransA) %then %do;
   data &AENarrative_OutData_domA;
         merge aesubs(keep = usubjid in = ina) domA(keep = &&&FFDomAU.VARS partial_&FFDomAU.DTC STDY);
         by usubjid;
         if ina;
         %if %sysfunc(findw(%upcase(&&&FFDomAU.VARS), &FFDomAU.DY)) %then %do;
            drop &FFDomAU.dy;
         %end;
         rename stdy = &FFDomAU.DY;
         if partial_&FFDomAU.DTC = 1 then &FFDomAU.DY = .;
         drop partial_&FFDomAU.DTC;
      run;
   %end;
   %else %if %upcase(&FTransColA) = TEST CODE %then %do;      
      %let FTransA = &FTransA STDY partial_&FFDomAU.DTC;
      
      proc sort data = domA out = domAt;
         by &FTransA;
      run;
   
      proc transpose data = domAt %if %upcase(&FDupsA) = YES %then %do; LET %end; out = domAt(drop = _name_ _label_);
         by &FTransA;
         %if %scan(%upcase(&StOrA), 1, " ") = ORIGINAL %then %do;
            var &FFDomAU.ORRES;
         %end;
         %else %do;
            %if %sysfunc(findw(%upcase(&&&FFDomAU.VARS), &FFDomAU.STRESN)) %then %do;
               var &FFDomAU.STRESN;
            %end;
            %else %if %sysfunc(findw(%upcase(&&&FFDomAU.VARS), &FFDomAU.STRESC)) %then %do;
               var &FFDomAU.STRESC;
            %end;
         %end;
         id &FFDomAU.TESTCD;
         idlabel &FVarA;
      run;
      
      data &AENarrative_OutData_domA;
         merge aesubs(keep = usubjid in = ina) domAt;
         by usubjid;
         if ina;
         %if %sysfunc(findw(%upcase(&FTransA), &FFDomAU.DY)) %then %do;
            drop &FFDomAU.dy;
         %end;         
         rename stdy = &FFDomAU.DY;
         if partial_&FFDomAU.DTC = 1 then &FFDomAU.DY = .;
         drop partial_&FFDomAU.DTC;
      run;
      
      proc datasets nolist;
         delete domAt;
      quit;
   %end; 
   %else %do;            
      %if ^%sysfunc(findw(%upcase(&FTransA), &FFDomAU.TEST)) and %sysfunc(findw(%upcase(&FTransA), &FFDomAU.TESTCD)) %then %do;
         %REPLACEWORD(where = FTransA, word = &FFDomAU.TESTCD, newword = &FFDomAU.TESTCD &FFDomAU.TEST);
      %end;
      
      %if ^%sysfunc(findw(%upcase(&FTransA), &FFDomAU.TESTCD)) and %sysfunc(findw(%upcase(&FTransA), &FFDomAU.TEST)) %then %do;
         %REPLACEWORD(where = FTransA, word = &FFDomAU.TEST, newword = &FFDomAU.TESTCD &FFDomAU.TEST);
      %end;      
      
      proc sort data = domA out = domAt;
         by &FTransA;
      run;
   
      proc transpose data = domAt prefix = &trVisit %if %upcase(&FDupsA) = YES %then %do; LET %end; out = domAt(drop = _name_ _label_);
         by &FTransA;
         %if %scan(%upcase(&StOrA), 1, " ") = ORIGINAL %then %do;
            var &FFDomAU.ORRES;
         %end;
         %else %do;
            %if %sysfunc(findw(%upcase(&&&FFDomAU.VARS), &FFDomAU.STRESN)) %then %do;
               var &FFDomAU.STRESN;
            %end;
            %else %if %sysfunc(findw(%upcase(&&&FFDomAU.VARS), &FFDomAU.STRESC)) %then %do;
               var &FFDomAU.STRESC;
            %end;
         %end;
         id VISITNUM;
      run;
      
      data &AENarrative_OutData_domA;
         merge aesubs(keep = usubjid in = ina) domAt;
      by usubjid;
      if ina;
   run;      

      proc datasets nolist;
         delete domAt;
      quit;      
%end;
%end;

%if %upcase(&FFlB) = YES %then %do;
   %LOADDATA(domlist = &FFDomBU,mergesupp=0, checkreview=0, _TempWhereStmt = %bquote(&FWhereStmtB), _NarrFlag = 1);    

   %NARRBYLIST(domain = &FFDomBU, code = B);

   proc datasets nolist;
      change &FFDomBU = domB;
   quit;

   data domB;
      set domB;
      %if %scan(%upcase(&StOrB), 1, " ") = ORIGINAL %then %do;
         if missing(&FFDomBU.ORRES) then delete;
      %end;
      %else %do;
         %if %sysfunc(findw(%upcase( &&&FFDomBU.VARS), &FFDomBU.STRESC)) and %sysfunc(findw(%upcase( &&&FFDomBU.VARS), &FFDomBU.STRESN)) %then %do;
            if missing(&FFDomBU.STRESC) and missing(&FFDomBU.STRESN) then delete;
         %end;
         %else %if %sysfunc(findw(%upcase( &&&FFDomBU.VARS), &FFDomBU.STRESC)) and ^%sysfunc(findw(%upcase( &&&FFDomBU.VARS), &FFDomBU.STRESN)) %then %do;
            if missing(&FFDomBU.STRESC) then delete;
         %end;
         %else %if ^%sysfunc(findw(%upcase( &&&FFDomBU.VARS), &FFDomBU.STRESC)) and %sysfunc(findw(%upcase( &&&FFDomBU.VARS), &FFDomBU.STRESN)) %then %do;
            if missing(&FFDomBU.STRESN) then delete;
         %end;       
      %end;
      %if &FFDomBU = LB %then %do;
         if missing(LBNRIND) then delete;
      %end;    
   run;
   %NObsVars(domB);
   %if ^&nobs %then %do;
      %put [MS_S_0043];
      %put ERROR: No observations satisfy &FWhereStmtB. Please relax criteria.;
      %let exiterror = 1;
      %goto exit;
   %end;
         
   %let AENarrative_OutData_domB =;
   %PrepOutDS(AENarrative_OutData_domB,OutDataName_domB, &Outname.&FFDomBU);
   %let includeds = &includeds &OutDataName_domB..sas7bdat,;

   *** all findings for subjects of interest with events of interest ***;
   %if ^%length(&FTransB) %then %do;
   data &AENarrative_OutData_domB;
         merge aesubs(keep = usubjid in = ina) domB(keep = &&&FFDomBU.VARS partial_&FFDomBU.DTC STDY);
         by usubjid;
         if ina;
         %if %sysfunc(findw(%upcase(&&&FFDomBU.VARS), &FFDomBU.DY)) %then %do;
            drop &FFDomBU.dy;
         %end;
         rename stdy = &FFDomBU.DY;
         if partial_&FFDomBU.DTC = 1 then &FFDomBU.DY = .;
         drop partial_&FFDomBU.DTC;
      run;
   %end;
   %else %if %upcase(&FTransColB) = TEST CODE %then %do;      
      %let FTransB = &FTransB STDY partial_&FFDomBU.DTC;
      
      proc sort data = domB out = domBt;
         by &FTransB;
      run;
   
      proc transpose data = domBt %if %upcase(&FDupsB) = YES %then %do; LET %end; out = domBt(drop = _name_ _label_);
         by &FTransB;
         %if %scan(%upcase(&StOrB), 1, " ") = ORIGINAL %then %do;
            var &FFDomBU.ORRES;
         %end;
         %else %do;
            %if %sysfunc(findw(%upcase(&&&FFDomBU.VARS), &FFDomBU.STRESN)) %then %do;
               var &FFDomBU.STRESN;
            %end;
            %else %if %sysfunc(findw(%upcase(&&&FFDomBU.VARS), &FFDomBU.STRESC)) %then %do;
               var &FFDomBU.STRESC;
            %end;
         %end;
         id &FFDomBU.TESTCD;
         idlabel &FVarB;
      run;
      
      data &AENarrative_OutData_domB;
         merge aesubs(keep = usubjid in = ina) domBt;
      by usubjid;
      if ina;
         %if %sysfunc(findw(%upcase(&FTransB), &FFDomBU.DY)) %then %do;
            drop &FFDomBU.dy;
         %end;         
         rename stdy = &FFDomBU.DY;
         if partial_&FFDomBU.DTC = 1 then &FFDomBU.DY = .;
         drop partial_&FFDomBU.DTC;
   run;   
      
      proc datasets nolist;
         delete domBt;
      quit;
%end;
   %else %do;            
      %if ^%sysfunc(findw(%upcase(&FTransB), &FFDomBU.TEST)) and %sysfunc(findw(%upcase(&FTransB), &FFDomBU.TESTCD)) %then %do;
         %REPLACEWORD(where = FTransB, word = &FFDomBU.TESTCD, newword = &FFDomBU.TESTCD &FFDomBU.TEST);
%end;

      %if ^%sysfunc(findw(%upcase(&FTransB), &FFDomBU.TESTCD)) and %sysfunc(findw(%upcase(&FTransB), &FFDomBU.TEST)) %then %do;
         %REPLACEWORD(where = FTransB, word = &FFDomBU.TEST, newword = &FFDomBU.TESTCD &FFDomBU.TEST);
      %end;      
      
      proc sort data = domB out = domBt;
         by &FTransB;
      run;
   
      proc transpose data = domBt prefix = &trVisit %if %upcase(&FDupsB) = YES %then %do; LET %end; out = domBt(drop = _name_ _label_);
         by &FTransB;
         %if %scan(%upcase(&StOrB), 1, " ") = ORIGINAL %then %do;
            var &FFDomBU.ORRES;
         %end;
         %else %do;
            %if %sysfunc(findw(%upcase(&&&FFDomBU.VARS), &FFDomBU.STRESN)) %then %do;
               var &FFDomBU.STRESN;
            %end;
            %else %if %sysfunc(findw(%upcase(&&&FFDomBU.VARS), &FFDomBU.STRESC)) %then %do;
               var &FFDomBU.STRESC;
            %end;
         %end;
         id VISITNUM;
      run;
      
      data &AENarrative_OutData_domB;
         merge aesubs(keep = usubjid in = ina) domBt;
         by usubjid;
         if ina;
      run;

      proc datasets nolist;
         delete domBt;
      quit;      
   %end;      
%end;

%if %upcase(&FFlC) = YES %then %do;
   %LOADDATA(domlist = &FFDomCU,mergesupp=0, checkreview=0, _TempWhereStmt = %bquote(&FWhereStmtC), _NarrFlag = 1);    

   %NARRBYLIST(domain = &FFDomCU, code = C);

   proc datasets nolist;
      change &FFDomCU = domC;
   quit;

   data domC;
      set domC;
      %if %scan(%upcase(&StOrC), 1, " ") = ORIGINAL %then %do;
         if missing(&FFDomCU.ORRES) then delete;
      %end;
      %else %do;
         %if %sysfunc(findw(%upcase( &&&FFDomCU.VARS), &FFDomCU.STRESC)) and %sysfunc(findw(%upcase( &&&FFDomCU.VARS), &FFDomCU.STRESN)) %then %do;
            if missing(&FFDomCU.STRESC) and missing(&FFDomCU.STRESN) then delete;
         %end;
         %else %if %sysfunc(findw(%upcase( &&&FFDomCU.VARS), &FFDomCU.STRESC)) and ^%sysfunc(findw(%upcase( &&&FFDomCU.VARS), &FFDomCU.STRESN)) %then %do;
            if missing(&FFDomCU.STRESC) then delete;
         %end;
         %else %if ^%sysfunc(findw(%upcase( &&&FFDomCU.VARS), &FFDomCU.STRESC)) and %sysfunc(findw(%upcase( &&&FFDomCU.VARS), &FFDomCU.STRESN)) %then %do;
            if missing(&FFDomCU.STRESN) then delete;
         %end;       
      %end;    
      %if &FFDomCU = LB %then %do;
         if missing(LBNRIND) then delete;
      %end;    
   run;
   %NObsVars(domC);
   %if ^&nobs %then %do;
      %put [MS_S_0044];
      %put ERROR: No observations satisfy &FWhereStmtC. Please relax criteria.;
      %let exiterror = 1;
      %goto exit;
   %end;
         
   %let AENarrative_OutData_domC =;
   %PrepOutDS(AENarrative_OutData_domC,OutDataName_domC, &Outname.&FFDomCU);
   %let includeds = &includeds &OutDataName_domC..sas7bdat,;

   *** all findings for subjects of interest with events of interest ***;
   %if ^%length(&FTransC) %then %do;
   data &AENarrative_OutData_domC;
         merge aesubs(keep = usubjid in = ina) domC(keep = &&&FFDomCU.VARS partial_&FFDomCU.DTC STDY);
         by usubjid;
         if ina;
         %if %sysfunc(findw(%upcase(&&&FFDomCU.VARS), &FFDomCU.DY)) %then %do;
            drop &FFDomCU.dy;
         %end;
         rename stdy = &FFDomCU.DY;
         if partial_&FFDomCU.DTC = 1 then &FFDomCU.DY = .;
         drop partial_&FFDomCU.DTC;
      run;
   %end;
   %else %if %upcase(&FTransColC) = TEST CODE %then %do;      
      %let FTransC = &FTransC STDY partial_&FFDomCU.DTC;
      
      proc sort data = domC out = domCt;
         by &FTransC;
      run;
   
      proc transpose data = domCt %if %upcase(&FDupsC) = YES %then %do; LET %end; out = domCt(drop = _name_ _label_);
         by &FTransC;
         %if %scan(%upcase(&StOrC), 1, " ") = ORIGINAL %then %do;
            var &FFDomCU.ORRES;
         %end;
         %else %do;
            %if %sysfunc(findw(%upcase(&&&FFDomCU.VARS), &FFDomCU.STRESN)) %then %do;
               var &FFDomCU.STRESN;
            %end;
            %else %if %sysfunc(findw(%upcase(&&&FFDomCU.VARS), &FFDomCU.STRESC)) %then %do;
               var &FFDomCU.STRESC;
            %end;
         %end;
         id &FFDomCU.TESTCD;
         idlabel &FVarC;
      run;
      
   data &AENarrative_OutData_domC;
         merge aesubs(keep = usubjid in = ina) domCt;
         by usubjid;
         if ina;
         %if %sysfunc(findw(%upcase(&FTransC), &FFDomCU.DY)) %then %do;
            drop &FFDomCU.dy;
         %end;         
         rename stdy = &FFDomCU.DY;
         if partial_&FFDomCU.DTC = 1 then &FFDomCU.DY = .;
         drop partial_&FFDomCU.DTC;
      run;
      
      proc datasets nolist;
         delete domCt;
      quit;
   %end; 
   %else %do;            
      %if ^%sysfunc(findw(%upcase(&FTransC), &FFDomCU.TEST)) and %sysfunc(findw(%upcase(&FTransC), &FFDomCU.TESTCD)) %then %do;
         %REPLACEWORD(where = FTransC, word = &FFDomCU.TESTCD, newword = &FFDomCU.TESTCD &FFDomCU.TEST);
      %end;
      
      %if ^%sysfunc(findw(%upcase(&FTransC), &FFDomCU.TESTCD)) and %sysfunc(findw(%upcase(&FTransC), &FFDomCU.TEST)) %then %do;
         %REPLACEWORD(where = FTransC, word = &FFDomCU.TEST, newword = &FFDomCU.TESTCD &FFDomCU.TEST);
      %end;      
      
      proc sort data = domC out = domCt;
         by &FTransC;
      run;
   
      proc transpose data = domCt prefix = &trVisit %if %upcase(&FDupsC) = YES %then %do; LET %end; out = domCt(drop = _name_ _label_);
         by &FTransC;
         %if %scan(%upcase(&StOrC), 1, " ") = ORIGINAL %then %do;
            var &FFDomCU.ORRES;
         %end;
         %else %do;
            %if %sysfunc(findw(%upcase(&&&FFDomCU.VARS), &FFDomCU.STRESN)) %then %do;
               var &FFDomCU.STRESN;
            %end;
            %else %if %sysfunc(findw(%upcase(&&&FFDomCU.VARS), &FFDomCU.STRESC)) %then %do;
               var &FFDomCU.STRESC;
            %end;
         %end;
         id VISITNUM;
      run;
      
      data &AENarrative_OutData_domC;
         merge aesubs(keep = usubjid in = ina) domCt;
      by usubjid;
      if ina;
   run;      

      proc datasets nolist;
         delete domCt;
      quit;      
%end;
%end;

************************************;
*** Data Modifications *************;
************************************;

data aework;
   %if %length(&saepreslist) %then %do; 
      length saetext $ 2000;
   %end;
   set aework;
   %if %length(&saepreslist) %then %do; 
      saetext = ""; 
      %let i = 1;
      %do %while(%klength(%kscan(&saepreslist,&i,' ')));
        %let tempSPLi = %kscan(&saepreslist,&i,' ');
        if (%unquote(%LocaleGetAll(Y_Text, _case=u, _indexv = tempSPLi))) then saetext = catx("~", saetext, "%sysfunc(lowcase(%scan(&saeprestextlist,&i,~)))");
       %let i = &i + 1;
      %end;
      *drop &SaePresList;
      *saetext = lower(saetext);
   %end;
run;
%NObsVars(aework);
%if &nobs %then %do; 
%SHRINKCHARS(dsn = aework, vars = saetext);
%end;

%if &doMH %then %do;
   data mh;
      length mhyear $ 20;
      set mh;
      %if %length(&MHSTDTC) %then %do;
         mhyear = substr(mhstdtc, 1, 4);
         if mhyear ne "" then mhyearsort = mhyear + 0;
         else if mhyear = "" then mhyearsort = 0;
         if mhyear = "" then mhyear = "&trStartUnknown"; 
         %if &MHOption = MHTERM %then %do;
            mhterm2 = catx(" ", &MHOption, cats("(",mhyear,")"));
         %end;
         %else %if &MHOption = MHDECOD %then %do;
            if mhdecod = "" then mhterm2 = catx(" ", mhterm, cats("(&trUncoded,",mhyear,")"));
            else mhterm2 = catx(" ", &MHOption, cats("(",mhyear,")"));
         %end;
         drop &MHSTDTC;
      %end;
      %else %do; 
         mhyearsort = 0;
         %if &MHOption = MHTERM %then %do;
            mhterm2 = &MHOption;
         %end;
         %else %if &MHOption = MHDECOD %then %do;
            if mhdecod = "" then mhterm2 = catx(" ", mhterm, "(&trUncoded)");
            else mhterm2 = &MHOption;
         %end; 
      %end;
   run;

   proc sort data = mh;
      by usubjid descending mhyearsort &MHOption;
   run;
%end;

%if &doDS %then %do;
   *** Taking last treatment disposition event as final trial disposition;    
   data ds(keep = usubjid dsterm dsdecod dsstdtcp dsstdy dsstdt dsstdtc);     
      %if %length(%unquote(&DSWhereStmt)) %then %do;
         set ds;
         where %unquote(&DSWhereStmt);
      %end;
      %else %do;
         %if %klength(&DSEPOCH) and %klength(&DSCAT) %then %do;
            set ds(where = (%unquote(%LocaleGetAll(Treatment_Text, _case=u, _indexv = DSEPOCH)) and
             kupcase(&DSCAT) in (%unquote(%LocaleGetAll(DispositionEvent_Text, _case = u)))));
         %end;
         %else %if %klength(&DSEPOCH) %then %do;
            set ds(where = (%unquote(%LocaleGetAll(Treatment_Text, _case=u, _indexv = DSEPOCH))));
         %end;       
         %else %if %klength(&DSCAT) %then %do;
            set ds(where = (%unquote(%LocaleGetAll(DispositionEvent_Text, _case=u, _indexv = DSCAT))));
         %end;
         %else %do;
            set ds;
         %end;         
      %end;
      by usubjid;
      if last.usubjid;
      %DATE9(var=dsstdtc,return=dsstdt);
      
      rename stdy = DSSTDY;
      
      if DSSTDTC ne "" then do;
         if d9partial_DSSTDTC then dsstdtcp = cats(put(dsstdt,&IntDateFormat..),"*");
         else dsstdtcp = put(dsstdt,&IntDateFormat..);   
      end;
   run;
%end;

%if &doCM %then %do;
   data cm;
      set cm(keep = usubjid rfstdtc cmtrt endy stdy &CMDECOD cmstdtc cmendtc &CMINDC &CMSEQ);
      if cmdecod = "" then cmdecod = "&trUncodedCM";
      %DATE9(var=cmstdtc,return=cmstdt);
      %DATE9(var=cmendtc,return=cmendt);
      *** if CM start or stop dates are missing or partial then flag ***;
      if d9partial_CMSTDTC or d9partial_CMENDTC or cmstdt = . or cmendt = . then &CMOption = cats(&CMOption,"*");
      
      rename endy = CMENDY stdy = CMSTDY;
   
      %if %upcase(&cmindfl) = Y and %length(&CMINDC) %then %do;
         if cmindc = "" then cmindc = "&trIndicationNotProv";
         cmterm2 = catx(" ", lowcase(&CMOption), cats("(",cmindc,")"));
      %end;
      %else %do;
         cmterm2 = &CMOption;
      %end;      
   run;
%end;

%if &doHO %then %do;
   data ho(keep = usubjid hostdt hoterm2);
      set ho;
      %if %length(&HODECOD) %then %do;
         if hodecod = "" then hodecod = "&trUncodedHO";
      %end;
      %DATE9(var=hostdtc,return=hostdt);
      *** if HO start or stop dates are missing or partial then flag ***;
      if d9partial_HOSTDTC then &HOOption = cats(&HOOption,"*");     

      rename stdy = HOSTDY;
      format hostdt date9.;
      if hostdt > . then hoterm2 = catx(" ", lowcase(&HOOption), cats("(",put(hostdt, date9.),","),cats(catx(" ","&trDay",STDY),")"));
      else hoterm2 = catx(" ", lowcase(&HOOption), "(Missing Start Date)");
   run;
%end;

************************************;
******* Findings *******************;
************************************;

%if %upcase(&FFlA) = YES %then %do;
   %FINDING(code = A);
   %if &exiterror %then %goto exit;
%end;

%if %upcase(&FFlB) = YES %then %do;
   %FINDING(code = B);
   %if &exiterror %then %goto exit;
%end;

%if %upcase(&FFlC) = YES %then %do;
   %FINDING(code = C);
   %if &exiterror %then %goto exit;
%end;

************************************;
******* Medical History ************;
************************************;

%if &doMH %then %do;
   data onemh(keep = usubjid mhtext);
      length mhtext $ 5000;
      set mh;
      retain mhtext;
      by usubjid;   
      if first.usubjid then mhtext = lowcase(mhterm2);
      else mhtext = catx("~", mhtext, lowcase(mhterm2));
      if last.usubjid;
   run;
   %NObsVars(onemh);
   %if &nobs %then %do; 
   %SHRINKCHARS(dsn = onemh, vars = mhtext);  
%end;
%end;

************************************;
******* Dosing Information *********;
************************************;

%if &doEX %then %do;
   %if %upcase(&EXENDTCFl) = YES %then %do;   
      %let _tid_ = %sysfunc(open(ex));
      %let vn = %sysfunc(varnum(&_tid_,exstdtc));
      %let StartLen = %sysfunc(varlength(&_tid_,&vn));
      %let _tid_ = %sysfunc(close(&_tid_));         
   %end;

   data ex(keep = USUBJID EXDOSE EXDOSU EXENDT exendy EXSTDT exstdy EXTRT);
      %if %upcase(&EXENDTCFl) = YES %then %do;
         length exendtc $ &StartLen;
      %end;
      set ex;
      %DATE9(var=exstdtc,return=exstdt);
      %if %upcase(&EXENDTCFl) = YES %then %do;
         exendtc = exstdtc;
         label exendtc = "&trEndDateTimeofTreatment";
         %let EXVARS = &EXVARS EXENDTC;
      %end;
      %DATE9(var=exendtc,return=exendt);     
      *** study days: use what is provided or calculate ***;
      %calculateStudyDays(EX);
      drop exstdy exendy log2_exdur exdur errdt:;
      rename endy = EXENDY stdy = EXSTDY;
   run;
   
   *** flag for subjects with any partial dates for narrative review ***;
   data exdoseimp(keep = usubjid);
      set ex;
      if d9partial_EXSTDTC or d9partial_EXENDTC;
   run;
   
   data exdoseimp;
      set exdoseimp;
      by usubjid;
      if last.usubjid;
   run;
      
   *** expand dose record to have each day represented as a single record and sum up across all records on day for each extrt ***;
   data ex;
      set ex;
      where exstdt > . and exendt > .;
      do date = exstdt to exendt;
         output;
      end;
      format date &IntDateFormat..;
   run;
   
   data ex;
      set ex;
      stdy = exstdy + (date - exstdt);
   run;

   proc means nway noprint data = ex;
      class usubjid extrt exdosu date stdy;
      var exdose;
      output out = ex(drop = _TYPE_  _FREQ_) sum = exdose;
   run;
   
   data ex;
      set ex;
      *** Keep records where drug is taken ***;
      *** assuming exdose = 0 is an interruption (an option for placebo-vehicle ***;
      where ((kupcase(extrt) not in (
       %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
       %unquote(%LocaleGetAll(Placebo_Text, _case = u))) and exdose > 0) or
       (kupcase(extrt) in (
       %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
       %unquote(%LocaleGetAll(Placebo_Text, _case = u))) and exdose &PVsign 0));
   run;
         
   proc sort data = ex out = exmeans;
      by usubjid date;
   run;
   
   data exmeans(keep = usubjid exstdt_min exendt_max exstdy_min exendy_max);
      set exmeans;      
      by usubjid date;
      retain exstdt_min exendt_max exstdy_min exendy_max;
      if first.usubjid then do;
         exstdt_min = date;
         exstdy_min = stdy;
      end;
      if last.usubjid then do;
         exendt_max = date;
         exendy_max = stdy;      
      end;
      if last.usubjid;
      format exstdt_min exendt_max &IntDateFormat..;
   run;
   
   *** all drugs on date of first dose ***;
   data dosefirst;
      merge ex exmeans(keep = usubjid exstdt_min);
      by usubjid;
      if date ne exstdt_min then delete;
   run;
      
   data dosefirst(keep = usubjid drugatfirstdose);
      length drugatfirstdose $ 500;
      set dosefirst; 
      by usubjid; 
      retain drugatfirstdose;       
      if first.usubjid then do;
         if (kupcase(extrt) in (
          %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
          %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then do;
            drugatfirstdose = trim(extrt);
         end;
         else do;
            drugatfirstdose = catx(" &trOf ",catx(" ", exdose, exdosu),trim(extrt));
         end;        
      end;
      else do;
         if (kupcase(extrt) in (
          %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
          %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then do;
            drugatfirstdose = catx("~",drugatfirstdose,trim(extrt));
         end;
         else do;
            drugatfirstdose = catx("~",drugatfirstdose,catx(" &trOf ",catx(" ", exdose, exdosu),trim(extrt)));
         end;              
      end;
      if last.usubjid;
   run;
   %NObsVars(dosefirst);
   %if &nobs %then %do;
      %SHRINKCHARS(dsn = dosefirst, vars = drugatfirstdose);  
   %end;

   *** all drugs on date of last dose ***;
   data doselast;
      merge ex exmeans(keep = usubjid exendt_max);
      by usubjid;
      if date ne exendt_max then delete;
   run;
      
   data doselast(keep = usubjid drugatlastdose);
      length drugatlastdose $ 500;
      set doselast; 
      by usubjid; 
      retain drugatlastdose;       
      if first.usubjid then do;
         if (kupcase(extrt) in (
          %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
          %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then do;
            drugatlastdose = trim(extrt);
         end;
         else do;
            drugatlastdose = catx(" &trOf ",catx(" ", exdose, exdosu),trim(extrt));
         end;        
      end;
      else do;
         if (kupcase(extrt) in (
          %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
          %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then do;
            drugatlastdose = catx("~",drugatlastdose,trim(extrt));
         end;
         else do;
            drugatlastdose = catx("~",drugatlastdose,catx(" &trOf ",catx(" ", exdose, exdosu),trim(extrt)));
         end;              
      end;
      if last.usubjid;
   run;
   %NObsVars(doselast);
   %if &nobs %then %do;
      %SHRINKCHARS(dsn = doselast, vars = drugatlastdose);  
   %end;

   data ex;
      merge ex exmeans;
      by usubjid;
   run;

   proc datasets nolist;
      delete exmeans;
   quit;

   proc sort data = ex;
      by usubjid extrt date exdose;
   run;

   *** determine how long a subject was at a dose for each drug before switching ***;
   data ex(drop = curr currdose date stdy);
      set ex;
      by usubjid extrt date exdose;
      retain dosestart curr currdose dosenum doseend;
      if first.extrt then do;
          dosestart = date;
          doseend = date;
          dosenum = 1;
          currdose = exdose;
          curr = date;
      end;
      else do;
         if (date - curr = 1) and ((kupcase(extrt) in (
           %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
           %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) or
          (kupcase(extrt) not in (
           %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
           %unquote(%LocaleGetAll(Placebo_Text, _case = u))) and exdose = currdose)) then do;
            curr = date;
            doseend = date;
         end;
         else do;
            dosestart = date;
            dosenum = dosenum + 1;
            curr = date;
            doseend = date;
            currdose = exdose;
         end;
      end;
      format dosestart doseend curr &IntDateFormat..;
   run;

   data ex;
      set ex;
      by usubjid extrt dosenum;
      if last.dosenum;
   run;
      
   *** Merge in imputation information to subjects with events ***;
   data aesubs;
      merge aesubs(in = ina) exdoseimp(in = inb);
      by usubjid;
      if ina;
      if inb then inexdoseimp = 1;
      else inexdoseimp = 0;
   run;  

   *** left join to code for people with no drug taken ***;
   proc sql;
      create table oneex(drop = usubjid1 usubjid2) as
      select coalesce(aework.usubjid1, ex.usubjid2) as usubjid, aework.*, ex.*
      from aework(rename=(usubjid= usubjid1) keep = usubjid aeseq aestdt _OTFLAG _OTFUFLAG _TRTEMRG) left join ex(rename=(usubjid= usubjid2))
      on aework.usubjid1 = ex.usubjid2 
      order by usubjid, aestdt, aeseq, extrt, dosestart;
   quit;
   
   *** Using minimum start date to determine if subject got dosed or not;
   data oneex(drop = _TRTEMRG _OTFLAG _OTFUFLAG);
      length flag $ 14;
      merge oneex dosefirst doselast;
      by usubjid;
      aedaysfromfirstdose = aestdt - exstdt_min;
      aedaysfromlastdose = aestdt - exendt_max; 

      *** Use TEAE Algorithm to get definitions ***;
      if exstdt_min = . then flag = "&trNoTreatment";
      else if _OTFLAG = 1 then flag = "&trOnTreatment";
      else if _OTFUFLAG = 1 then flag = "&trPostTreatment";      
      else if _TRTEMRG = 0 then flag = "&trPreTreatment";
   run;

   data cumdose;
      set oneex(where = (dosestart <= aestdt));
      cumdose = (min(aestdt,doseend) - dosestart + 1) * exdose;
   run;
   
   proc means nway noprint data = cumdose;
      class usubjid aeseq extrt exdosu;
      var cumdose;
      output out = cumdose(drop = _TYPE_  _FREQ_) sum = cumdose;
   run;   

   data cumdose(keep = usubjid aeseq cumdrugatevent);
      length cumdrugatevent $ 500;
      set cumdose;
      by usubjid aeseq;
      retain cumdrugatevent;
      if first.aeseq then do;
         if (kupcase(extrt) in (
          %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
          %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then do;
            cumdrugatevent = trim(extrt);
         end;
         else do;
            cumdrugatevent = catx(" &trOf ", catx(" ", cumdose, exdosu),trim(extrt));
         end;
      end;
      else do;
         if (kupcase(extrt) in (
          %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
          %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then do;
            cumdrugatevent = catx("~", cumdrugatevent, trim(extrt));
         end;
         else do;
            cumdrugatevent = catx("~", cumdrugatevent, catx(" &trOf ", catx(" ", cumdose, exdosu),trim(extrt)));
         end;
      end;
      if last.aeseq;
   run;
   %NObsVars(cumdose);
   %if &nobs %then %do;
      %SHRINKCHARS(dsn = cumdose, vars = cumdrugatevent);
   %end;   

   *** if all drugs taken at least once per day, determine drugs at event and how long at that dose ***;
   %if &EXDaily = Y %then %do;   
      proc sort data = oneex(where = (dosestart <= aestdt <= doseend)) out = onetrt;
         by usubjid aeseq;
      run;

      data onetrt;
         length drugatevent $ 500 numdaysdrugattime $ 50;
         set onetrt;
         by usubjid aeseq;
         retain drugatevent numdaysdrugattime;
         if first.aeseq then do;
            if (kupcase(extrt) in (
             %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
             %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then do;
               drugatevent = trim(extrt);
            end;
            else do;
               drugatevent = catx(" &trOf ", catx(" ", exdose, exdosu),trim(extrt));
            end;
            numdaysdrugattime = trim(left(put(aestdt - dosestart + 1, 8.0)));
         end;
         else do;
            if (kupcase(extrt) in (
             %unquote(%LocaleGetAll(Vehicle_Text, _case = u)),
             %unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then do;
               drugatevent = catx("~", drugatevent, trim(extrt));
            end;
            else do;
               drugatevent = catx("~", drugatevent, catx(" &trOf ", catx(" ", exdose, exdosu),trim(extrt)));
            end;
            numdaysdrugattime = catx("~", numdaysdrugattime, trim(left(put(aestdt - dosestart + 1, 8.0))));    
         end;
         if last.aeseq;
      run;
      %NObsVars(onetrt);
      %if &nobs %then %do;
      %SHRINKCHARS(dsn = onetrt, vars = drugatevent numdaysdrugattime); 
      %end;

      proc sort data = oneex(keep = usubjid aeseq drugatfirstdose drugatlastdose aedaysfromfirstdose aedaysfromlastdose exstdt_min exendt_max exstdy_min exendy_max flag) out = oneex;
         by usubjid aeseq;
      run;

      data oneex;
         merge oneex onetrt(keep = usubjid aeseq drugatevent numdaysdrugattime) cumdose;
         by usubjid aeseq;
         if first.aeseq;
         if (kupcase(cumdrugatevent) in (%unquote(%LocaleGetAll(Vehicle_Text, _case = u)),%unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then cumdrugatevent = "";
      run;  
   %end;
   %else %do;
      proc sort data = oneex(keep = usubjid aeseq drugatfirstdose drugatlastdose aedaysfromfirstdose aedaysfromlastdose exstdt_min exendt_max exstdy_min exendy_max flag) out = oneex;
         by usubjid aeseq;
      run;

      data oneex;
         merge oneex cumdose;
         by usubjid aeseq;
         if first.aeseq;
         if (kupcase(cumdrugatevent) in (%unquote(%LocaleGetAll(Vehicle_Text, _case = u)),%unquote(%LocaleGetAll(Placebo_Text, _case = u)))) then cumdrugatevent = "";
      run;   
   %end;
%end;

************************************;
******* Concomitant Medications ****;
************************************;

%if &doCM %then %do; 
   proc sql;
      create table onecm(drop = usubjid1 usubjid2) as
      select coalesce(aework.usubjid1, cm.usubjid2) as usubjid, aework.*, cm.* 
      from aework(rename=(usubjid = usubjid1)), cm(rename=(usubjid = usubjid2))
      where aework.usubjid1 = cm.usubjid2 
      order by usubjid, aeseq, cmstdt;
   quit;

   data onecm(where = (keep = 1));
      set onecm;
      if cmstdt > . and cmendt > . then do;
         if (cmstdt <= aestdt <= cmendt + &CMWindow) then keep = 1;
      end;
      else if cmstdt > . and cmendt = . then do;
         if (aestdt >= cmstdt) then keep = 1;
      end;
      else if cmstdt = . and cmendt > . then do;
         if (aestdt <= cmendt + &CMWindow) then keep = 1;
      end;
      else if cmstdt = . and cmendt = . then do;
         keep = 1;
      end;
   run;

   *** keep distinct medications-indications for each event ***;
   options minoperator mindelimiter=',';
   proc sort nodupkey data = onecm;
      %if %kupcase(&cmindfl) in (%unquote(%LocaleGetAll(Y_Text, _case = u, _quote = n))) and %length(&cmindc) %then %do;
         by usubjid aeseq &CMOption cmindc;
      %end;
      %else %do;
         by usubjid aeseq &CMOption;
      %end;
   run;
   options nominoperator;
   
   *** if a med-ind combo has partial and complete date, remove the partial date record ***;
   data onecm1 onecm2;
      set onecm(keep = usubjid aeseq &CMOption &cmindc cmterm2);
      if substr(&CMOption,length(&CMOption),1) = "*" then output onecm2;
      else output onecm1;
   run;
   
   data onecm2;
      set onecm2;
      &CMOption = substr(&CMOption,1,length(&CMOption)-1);
   run;
   
   proc sort data = onecm2;
      by usubjid aeseq &CMOption;
   run;
   
   options minoperator mindelimiter=',';
   data onecm2;
      merge onecm2(in = ina) onecm1(in = inb drop = cmterm2);
      %if %kupcase(&cmindfl) in (%unquote(%LocaleGetAll(Y_Text, _case = u, _quote = n))) and %klength(&cmindc) %then %do;
         by usubjid aeseq &CMOption cmindc;
      %end;
      %else %do;
         by usubjid aeseq &CMOption;
      %end;
      if ina and not inb;
   run;
   options nominoperator;
   
   data onecm;
      set onecm1 onecm2;
   run;
   
   proc datasets nolist;
      delete onecm1 onecm2;
   quit;

   proc sort data = onecm;
      %if %upcase(&cmindfl) = Y and %length(&cmindc) %then %do;
         by usubjid aeseq &CMOption cmindc;
      %end;
      %else %do;
         by usubjid aeseq &CMOption;
      %end;
   run;   

   data onecm(keep = usubjid aeseq cmtext);
      length cmtext $ 5000;
      set onecm;
      retain cmtext;
      by usubjid aeseq;
      if first.aeseq then cmtext = lowcase(cmterm2);
      else cmtext = catx("~", cmtext, lowcase(cmterm2));
      if last.aeseq;
   run;  
   %NObsVars(onecm);
   %if &nobs %then %do; 
   %SHRINKCHARS(dsn = onecm, vars = cmtext);       
%end;
%end;

*******************************************;
******* Nearby Adverse Events *************;
*******************************************;

proc sql;
   create table nearae(drop = usubjid1 usubjid2) as
   select coalesce(aework.usubjid1, ae.usubjid2) as usubjid, aework.*, ae.* 
   from aework(rename=(usubjid= usubjid1)), 
        ae(rename = (usubjid = usubjid2 aeseq = aeseq2 aestdt = aestdt2 aedecod = aedecod2 aestdtcp = aestdtcp2 %if %length(&AeSev) %then %do; &AeSev = &AeSev2 %end; aeterm2 = aeterm3))
   where (aework.usubjid1 = ae.usubjid2) 
   order by usubjid, aeseq, aeseq2;
quit;

*** drop the event itself from the nearby aes using the where statement ***;
data nearae(where = (keep = 1));
   set nearae(where = (aeseq ne aeseq2));   
   if aestdt - &AeWindow <= aestdt2 <= aestdt + &AeWindow then keep = 1;
   if index(aestdtcp2, "*") then do;
      aedecod2 = cats(aedecod2,"*");
      %if &AeSev = AESEV %then %do;
         aeterm3 = catx(" ", lowcase(aedecod2), cats("(",lowcase(&AeSev2),")"));
      %end;
      %else %if &AeSev = AETOXGR %then %do;
         aeterm3 = catx(" ", lowcase(aedecod2), cats("(",catx(" ", "&trGrade", lowcase(&AeSev2)),")"));
      %end;
      %else %do;
         aeterm3 = lowcase(aedecod2);
      %end;
   end;
run;

*** get unique event-severity pairs ***;
proc sort nodupkey data = nearae;
   by usubjid aeseq aedecod2 &AESEV2;
run;

*** if a event-severity combo has partial and complete date, remove the partial date record ***;
data nearae1 nearae2;
   set nearae(keep = usubjid aeseq aedecod2 &AESEV2 aeterm3);
   if substr(aedecod2,length(aedecod2),1) = "*" then output nearae2;
   else output nearae1;
run;

data nearae2;
   set nearae2;
   aedecod2 = substr(aedecod2,1,length(&CMOption)-1);
run;

proc sort data = nearae2;
   by usubjid aeseq aedecod2 &AESEV2;
run;

data nearae2;
   merge nearae2(in = ina) nearae1(in = inb drop = aeterm3);
   by usubjid aeseq aedecod2 &AESEV2;
   if ina and not inb;
run;

data nearae;
   set nearae1 nearae2;
run;

proc datasets nolist;
   delete nearae1 nearae2;
quit;

proc sort data = nearae;
   by usubjid aeseq aedecod2 &AESEV2;
run;

data nearae(keep = usubjid aeseq nearaetext);
   length nearaetext $ 5000;
   set nearae;
   retain nearaetext;
   by usubjid aeseq;
   if first.aeseq then nearaetext = aeterm3;
   else nearaetext = catx("~", nearaetext, aeterm3);
   if last.aeseq;
run;
%NObsVars(nearae);
%if &nobs %then %do; 
%SHRINKCHARS(dsn = nearae, vars = nearaetext);
%end;

************************************;
******* HealthCare Encounters ******;
************************************;

%if &doHO %then %do;
   %if &HOWin = ALL %then %do;     
      data oneho(keep = usubjid hotext);
         length hotext $ 5000;
         set ho;
         retain hotext;
         by usubjid;   
         if first.usubjid then hotext = hoterm2;
         else hotext = catx("~", hotext, hoterm2);
         if last.usubjid;
      run;
      %NObsVars(oneho);
      %if &nobs %then %do; 
      %SHRINKCHARS(dsn = oneho, vars = hotext);    
   %end;
   %end;
   %else %do;
      proc sql;
         create table nearho(drop = usubjid1 usubjid2) as
         select coalesce(aework.usubjid1, ho.usubjid2) as usubjid, aework.*, ho.* 
         from aework(rename=(usubjid = usubjid1)), ho(rename=(usubjid = usubjid2))
         where (aework.usubjid1 = ho.usubjid2) 
         order by usubjid, aeseq, hostdt;
      quit;

      *** drop the event itself from the nearby aes using the where statement ***;
      data nearho(where = (keep = 1));
         set nearho;   
         if aestdt - &HOWindow <= hostdt <= aestdt + &HOWindow then keep = 1;
      run;

      data nearho(keep = usubjid aeseq nearhotext);
         length nearhotext $ 5000;
         set nearho;
         retain nearhotext;
         by usubjid aeseq;
         if first.aeseq then nearhotext = hoterm2;
         else nearhotext = catx("~", nearhotext, hoterm2);
         if last.aeseq;
      run;         
      %NObsVars(nearho);
      %if &nobs %then %do; 
      %SHRINKCHARS(dsn = nearho, vars = nearhotext);    
   %end;
%end;
%end;

************************************;
******* Con Meds for the Event *****;
************************************;

%if &doCM %then %do;
   %if &RelRecCM = N %then %do;
   %if %length(&aecontrt) and %length(&cmindc) %then %do;
      proc sql;
         create table presccm(drop = usubjid1 usubjid2) as
         select coalesce(aework.usubjid1, cm.usubjid2) as usubjid, aework.*, cm.* 
         from aework(rename=(usubjid= usubjid1)), cm(rename=(usubjid= usubjid2))
         where aework.usubjid1 = cm.usubjid2 
         order by usubjid, aeseq, cmstdt;
      quit;

      *** pick conmeds where indication matches ae text and conmed date is same day or after ***;
      data presccm;
            set presccm(where = ((%unquote(%LocaleGetAll(Y_Text, _case=u, _indexv = aecontrt, _indexvamp = 0)))));
         if upcase(trim(aeterm)) = upcase(trim(cmindc)) and cmstdt >= aestdt;
      run;

      proc sort nodupkey data = presccm;
         by usubjid aeseq &CMOption cmindc;
      run;

      *** if a med-ind combo has partial and complete date, remove the partial date record ***;
      data presccm1 presccm2;
         set presccm(keep = usubjid aeseq &CMOption &cmindc cmstdt);
         if substr(&CMOption,length(&CMOption),1) = "*" then output presccm2;
         else output presccm1;
      run;

      data presccm2;
         set presccm2;
         &CMOption = substr(&CMOption,1,length(&CMOption)-1);
      run;
      
      proc sort data = presccm2;
         by usubjid aeseq &CMOption cmindc;
      run;

      data presccm2;
         merge presccm2(in = ina) presccm1(in = inb);
         by usubjid aeseq &CMOption cmindc;
         if ina and not inb;
      run;

      data presccm;
         set presccm1 presccm2(in = inb);
         if inb then &CMOption = cats(&CMOption,"*");
      run;

      proc datasets nolist;
         delete presccm1 presccm2;
      quit;

      proc sort data = presccm;
         by usubjid aeseq cmstdt &CMOption;
      run;
      
      data presccm(keep = usubjid aeseq presccmtext);
         length presccmtext $ 5000;
         set presccm;
         retain presccmtext;
         by usubjid aeseq;
         *** changed cmterm2 to &CMOption so indication is not included, which would match event ***;
         if first.aeseq then presccmtext = lowcase(&CMOption);
         else presccmtext = catx("~", presccmtext, lowcase(&CMOption));
         if last.aeseq;
      run;
      %NObsVars(presccm);
      %if &nobs %then %do; 
      %SHRINKCHARS(dsn = presccm, vars = presccmtext);
   %end;
%end;
%end;
   %else %if &RelRecCM = Y %then %do;
      %RELRECIT(basedom = AE, basevar = AESEQ, fromdom = CM, fromvar = CMSEQ);
      %if &exiterror %then %goto exit;

      proc sql;
         create table relrecAECM2(drop = usubjid1 usubjid2 aeseq1 aeseq2) as
         select coalesce(relrecAECM.usubjid1, aework.usubjid2) as usubjid, coalesce(relrecAECM.aeseq1, aework.aeseq2) as aeseq, relrecAECM.*, aework.* 
         from aework(keep = usubjid aeseq aecontrt rename = (usubjid = usubjid2 aeseq = aeseq2) where = (%unquote(%LocaleGetAll(Y_Text, _case=u, _indexv = aecontrt, _indexvamp = 0)))) 
              left join relrecAECM(rename = (usubjid = usubjid1 aeseq = aeseq1)) 
         on relrecAECM.usubjid1 = aework.usubjid2 and relrecAECM.aeseq1 = aework.aeseq2;
      quit;
      
      proc sql;
         create table presccm(drop = usubjid1 usubjid2 cmseq1 cmseq2) as
         select coalesce(relrecAECM2.usubjid1, cm.usubjid2) as usubjid, coalesce(relrecAECM2.cmseq1, cm.cmseq2) as cmseq, relrecAECM2.*, cm.* 
         from relrecAECM2(rename = (usubjid = usubjid1 cmseq = cmseq1)) 
              left join cm(keep = usubjid cmseq cmstdtc &CMOption rename = (usubjid = usubjid2 cmseq = cmseq2)) 
         on relrecAECM2.usubjid1 = cm.usubjid2 and relrecAECM2.cmseq1 = cm.cmseq2
         order by usubjid, aeseq, &CMOption;
      quit;

      *** if a med has partial and complete date, remove the partial date record ***;
      data presccm1 presccm2;
         set presccm(keep = usubjid aeseq &CMOption cmstdtc);
         if substr(&CMOption,length(&CMOption),1) = "*" then output presccm2;
         else output presccm1;
      run;

      data presccm2;
         set presccm2;
         &CMOption = substr(&CMOption,1,length(&CMOption)-1);
      run;

      proc sort data = presccm2;
         by usubjid aeseq &CMOption;
      run;

      data presccm2;
         merge presccm2(in = ina) presccm1(in = inb);
         by usubjid aeseq &CMOption;
         if ina and not inb;
      run;

      data presccm;
         set presccm1 presccm2(in = inb);
         if inb then &CMOption = cats(&CMOption,"*");
      run;

      proc datasets nolist;
         delete presccm1 presccm2;
      quit;

      proc sort data = presccm;
         by usubjid aeseq cmstdtc &CMOption;
      run;

      data presccm(keep = usubjid aeseq presccmtext);
         length presccmtext $ 5000;
         set presccm;
         retain presccmtext;
         by usubjid aeseq;
         if first.aeseq then presccmtext = lowcase(&CMOption);
         else presccmtext = catx("~", presccmtext, lowcase(&CMOption));
         if last.aeseq;
      run;
      %NObsVars(presccm);
      %if &nobs %then %do; 
         %SHRINKCHARS(dsn = presccm, vars = presccmtext);
      %end;              
   %end;
%end;

************************************;
***** Compile Report ***************;
************************************;

%if (&doHO and &HOWin = ALL) or &doMH or &doDS or &doCM %then %do;
   data aesubs;
      merge aesubs
      %if &doHO and &HOWin = ALL %then %do;
         oneho 
      %end;
      %if &doMH %then %do; onemh %end; 
      %if &doDS %then %do; ds %end;
      %if &doCM %then %do; totalcmcnt %end;
      ;
      by usubjid;
   run;
%end;

%if (&doHO  and &HOWin = ALL) or &doMH or &doDS or &doCM %then %do;
   proc datasets nolist;
      delete
      %if &doHO and &HOWin = ALL %then %do;
         oneho 
      %end;
      %if &doMH %then %do; onemh %end; 
      %if &doDS %then %do; ds %end;
      %if &doCM %then %do; totalcmcnt %end;
   ;
   quit;
%end;
   
proc sort data = aework;
   by usubjid aeseq;
run;
   
data aework;
   merge aework(in = ina)
   nearae
   %if &doEX %then %do; oneex %end;
   %if &doCM %then %do; onecm %end;
   %if &doCM and %length(&aecontrt) and %length(&cmindc) %then %do; presccm %end;
   %if &doHO and &HOWin = WIN %then %do; nearho %end; 
   %if %upcase(&FFlA) = YES and %upcase(&FBaseA) = YES  %then %do; baseA %end;
   %if %upcase(&FFlA) = YES and (%scan(%upcase(&FTypeA),1," ") = PRIOR or %scan(%upcase(&FTypeA),1," ") = BOTH) %then %do; preA %end;
   %if %upcase(&FFlA) = YES and (%scan(%upcase(&FTypeA),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeA),1," ") = BOTH) %then %do; postA %end;
   %if %upcase(&FFlB) = YES and %upcase(&FBaseB) = YES  %then %do; baseB %end;
   %if %upcase(&FFlB) = YES and (%scan(%upcase(&FTypeB),1," ") = PRIOR or %scan(%upcase(&FTypeB),1," ") = BOTH) %then %do; preB %end;
   %if %upcase(&FFlB) = YES and (%scan(%upcase(&FTypeB),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeB),1," ") = BOTH) %then %do; postB %end;
   %if %upcase(&FFlC) = YES and %upcase(&FBaseC) = YES  %then %do; baseC %end;
   %if %upcase(&FFlC) = YES and (%scan(%upcase(&FTypeC),1," ") = PRIOR or %scan(%upcase(&FTypeC),1," ") = BOTH) %then %do; preC %end;
   %if %upcase(&FFlC) = YES and (%scan(%upcase(&FTypeC),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeC),1," ") = BOTH) %then %do; postC %end;
   %if &AEACNMULT or &AERELMULT %then %do; aemult %end;
   ;
   by usubjid aeseq;
   if ina;
run;

proc datasets nolist;
   delete nearae
   %if &doEX  %then %do; oneex %end;
   %if &doCM  %then %do; onecm %end;
   %if &doCM and %length(&aecontrt) and %length(&cmindc) %then %do; presccm %end;
   %if &doHO and &HOWin = WIN %then %do; nearho %end; 
   %if %upcase(&FFlA) = YES and %upcase(&FBaseA) = YES  %then %do; baseA %end;
   %if %upcase(&FFlA) = YES and (%scan(%upcase(&FTypeA),1," ") = PRIOR or %scan(%upcase(&FTypeA),1," ") = BOTH) %then %do; preA %end;
   %if %upcase(&FFlA) = YES and (%scan(%upcase(&FTypeA),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeA),1," ") = BOTH) %then %do; postA %end;
   %if %upcase(&FFlB) = YES and %upcase(&FBaseB) = YES  %then %do; baseB %end;
   %if %upcase(&FFlB) = YES and (%scan(%upcase(&FTypeB),1," ") = PRIOR or %scan(%upcase(&FTypeB),1," ") = BOTH) %then %do; preB %end;
   %if %upcase(&FFlB) = YES and (%scan(%upcase(&FTypeB),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeB),1," ") = BOTH) %then %do; postB %end;
   %if %upcase(&FFlC) = YES and %upcase(&FBaseC) = YES  %then %do; baseC %end;
   %if %upcase(&FFlC) = YES and (%scan(%upcase(&FTypeC),1," ") = PRIOR or %scan(%upcase(&FTypeC),1," ") = BOTH) %then %do; preC %end;
   %if %upcase(&FFlC) = YES and (%scan(%upcase(&FTypeC),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeC),1," ") = BOTH) %then %do; postC %end;
   %if &AEACNMULT or &AERELMULT %then %do; aemult %end;
   ;
quit;

%if %length(&INVNAM) %then %do;
   %let InvestLabel = INVNAM;
%end;
%else %if %length(&INVID) %then %do;
   %let InvestLabel = INVID;
%end;
%else %do;
   %let InvestLabel = SITEID;
%end;
%CheckLabel(aework,InvestLabel);

data eventtext;
   length invhead aesevtxt aereltxt aeacntxt aeacnothtxt $ 50 exstdt_mintxt $ 9
          aestdaytxt aeendaytxt exstdaytxt dsstdaytxt $ 20 %if ^%length(&AGEU) %then %do; ageu $ 5 %end; %if ^&doEX %then %do; flag $ 14 %end; 
          PredaytxtA PostdaytxtA PredaytxtB PostdaytxtB PredaytxtC PostdaytxtC $ 20 
          ;
   *** will add findings day text above ***; 
   merge aework aesubs;
   by usubjid;

   %if %upcase(&SUBJIDFl) = YES and %length(&SUBJID) %then %do;
      subjectID = subjid;   
   %end;
   %else %do;
      subjectID = usubjid;         
   %end;

   aedecod = lowcase(aedecod);
   aeterm = lowcase(aeterm);
   
   %if %length(&AGEU) %then %do;
      ageu = lowcase(ageu);
      if substr(ageu, length(ageu), 1) = "s" then ageu = substr(ageu, 1, length(ageu)-1);
   %end;
   %else %do;
      ageu = "%sysfunc(lowcase(&trYear))";
   %end;
   invhead = "&InvestLabel";
   %if %length(&INVNAM) %then %do;
      rename invnam = invheadtxt;
   %end;
   %else %if %length(&INVID) %then %do;
      rename invid = invheadtxt;
   %end;
   %else %do;
      rename siteid = invheadtxt;
   %end;

   %if &AESEV = AESEV %then %do;
      aesevtxt = cats("(", lowcase(aesev), ")");
      drop aesev;
   %end;
   %else %if &AESEV = AETOXGR %then %do;
      aesevtxt = cats("(", catx(" ", "&trGrade", lowcase(aetoxgr)),")");
      drop aetoxgr;
   %end;
   %else %do;
      aesevtxt = "";
   %end;

   %if &doCM %then %do;
      if missing(totalcmcnt) then totalcmcnt = 0;
   %end;
   
   racetxt = lowcase(race);
   if findw(racetxt, "%sysfunc(lowcase(&trAsian))", " /") then racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trAsian))", "&trAsian");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trAfrican))", "&trAfrican");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trAmerican))", "&trAmerican");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trCaucasian))", "&trCaucasian");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trIndian))", "&trIndian");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trOriental))", "&trOriental");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trAlaska))", "&trAlaska");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trNative))", "&trNative");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trHawaiian))", "&trHawaiian");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trIslander))", "&trIslander");
   racetxt = tranwrd(racetxt, "%sysfunc(lowcase(&trPacific))", "&trPacific");
   drop race;
   
   *** using calculatestudyday() **********************;
   *** aestartday is replaced by aestdy ***************;
   *** dsstartday is replaced by dsstdy ***************;
   *** aeendday is replaced by aeendy *****************;
   *** firstmedstartday is replaced by exstdy_min *****;   
    
   if aestdy > . then aestdaytxt = catx(" ", "(&trDay", cats(aestdy,")"));
   else aestdaytxt = "";
   if aeendy > . then aeendaytxt = catx(" ", "(&trDay", cats(aeendy,")"));
   else aeendaytxt = "";
   
   %if &doDS %then %do;
      if dsstdy > . then dsstdaytxt = catx(" ", "(&trDay", cats(dsstdy,")"));
      else dsstdaytxt = "";
      dsterm = lowcase(dsterm);
   %end;
   %else %do;
      drop dsstdaytxt;
   %end;

   %if &doEX %then %do;
      if exstdy_min > . then exstdaytxt = catx(" ", "(&trDay", cats(exstdy_min,")"));      
      else exstdaytxt = "";
      if exstdt_min > . then exstdt_mintxt = put(exstdt_min,&IntDateFormat..);
      else exstdt_mintxt = "";

      if exendy_max > . then exendaytxt = catx(" ", "(&trDay", cats(exendy_max,")"));      
      else exendaytxt = "";
      if exendt_max > . then exendt_maxtxt = put(exendt_max,&IntDateFormat..);
      else exendt_maxtxt = "";
      
      if inexdoseimp then flagtxt = cats(flag,"*");
      else flagtxt = flag;      
   %end;
   %else %do;
      drop exstdaytxt exstdt_mintxt;
      if kupcase(arm) = "SCREEN FAILURE" or kupcase(arm) = "SCRNFAIL" or kupcase(arm) in (%unquote(%LocaleGetAll(ScreenFailure_Text, _case = u))) 
         or kupcase(arm) in (%unquote(%LocaleGetAll(NotTreated_Text, _case = u))) or  kupcase(arm) = "NOTTRT"
         or kupcase(arm) in (%unquote(%LocaleGetAll(NotAssigned_Text, _case = u))) or  kupcase(arm) = "NOTASSGN"      
      %if %klength(&ACTARM) %then %do;
         or kupcase(actarm) = "SCREEN FAILURE" or kupcase(actarm) = "SCRNFAIL" or kupcase(actarm) in (%unquote(%LocaleGetAll(ScreenFailure_Text, _case = u))) 
         or kupcase(actarm) in (%unquote(%LocaleGetAll(NotTreated_Text, _case = u))) or  kupcase(actarm) = "NOTTRT"
         or kupcase(actarm) in (%unquote(%LocaleGetAll(NotAssigned_Text, _case = u))) or  kupcase(actarm) = "NOTASSGN"
      %end;
         then flag = "&trNoTreatment";    
      else if _OTFLAG = 1 then flag = "&trOnTreatment";
      else if _OTFUFLAG = 1 then flag = "&trPostTreatment";
      else if _TRTEMRG = 0 then flag = "&trPreTreatment";      
      flagtxt = flag;
   %end;
   
   %if %length(&AEACN) %then %do;
      if kupcase(aeacn) in (
       %unquote(%LocaleGetAll(N_Text, _case = u)),
       %unquote(%LocaleGetAll(No_Text, _case = u)),
       %unquote(%LocaleGetAll(Not_Text, _case = u)),
       %unquote(%LocaleGetAll(None_Text, _case = u))) then aeacntxt = "%sysfunc(lowcase(&trNo))";
      else if kupcase(aeacn) in (
       %unquote(%LocaleGetAll(Y_Text, _case = u)),
       %unquote(%LocaleGetAll(Yes_Text, _case = u))) then aeacntxt = "%sysfunc(lowcase(&trYes))";
      else if kupcase(aeacn) ne "" then aeacntxt = lowcase(aeacn);
   %end;
   %else %do;
      drop aeacntxt;   
   %end;
   %if %length(&AEACNOTH) %then %do;
      if kupcase(aeacnoth) in (
       %unquote(%LocaleGetAll(N_Text, _case = u)),
       %unquote(%LocaleGetAll(No_Text, _case = u)),
       %unquote(%LocaleGetAll(Not_Text, _case = u)),
       %unquote(%LocaleGetAll(None_Text, _case = u))) then aeacnothtxt = "%sysfunc(lowcase(&trNo))";
      else if kupcase(aeacnoth) in (
       %unquote(%LocaleGetAll(Y_Text, _case = u)),
       %unquote(%LocaleGetAll(Yes_Text, _case = u))) then aeacnothtxt = "%sysfunc(lowcase(&trYes))";
      else if kupcase(aeacnoth) ne "" then aeacnothtxt = lowcase(aeacnoth);
   %end;
   %else %do;
      drop aeacnothtxt;   
   %end;
   %if %length(&AEREL) %then %do;
      if kupcase(aerel) in (
       %unquote(%LocaleGetAll(Y_Text, _case = u)),
       %unquote(%LocaleGetAll(Yes_Text, _case = u))) then aereltxt = "%sysfunc(lowcase(&trRelated))";
      else if kupcase(aerel) in (
       %unquote(%LocaleGetAll(N_Text, _case = u)),
       %unquote(%LocaleGetAll(No_Text, _case = u)),
       %unquote(%LocaleGetAll(Not_Text, _case = u)),
       %unquote(%LocaleGetAll(None_Text, _case = u))) then aereltxt = "%sysfunc(lowcase(&trNotRelated))";
      else if kupcase(aerel) in (
       %unquote(%LocaleGetAll(Unlikely_Text, _case = u))) then aereltxt = "%sysfunc(lowcase(&trUnlikelyRelated))";
      else if kupcase(aerel) in (
       %unquote(%LocaleGetAll(Probable_Text, _case = u)),
       %unquote(%LocaleGetAll(Probably_Text, _case = u))) then aereltxt = "%sysfunc(lowcase(&trProbablyRelated))";
      else if kupcase(aerel) in (
       %unquote(%LocaleGetAll(Possible_Text, _case = u)),
       %unquote(%LocaleGetAll(Possibly_Text, _case = u))) then aereltxt = "%sysfunc(lowcase(&trPossiblyRelated))";
      else if kupcase(aerel) in (
       %unquote(%LocaleGetAll(Definite_Text, _case = u)),
       %unquote(%LocaleGetAll(Definitely_Text, _case = u))) then aereltxt = "%sysfunc(lowcase(&trDefinitelyRelated))";
      else if kupcase(aerel) in (
       %unquote(%LocaleGetAll(Remote_Text, _case = u)),
       %unquote(%LocaleGetAll(Remotely_Text, _case = u))) then aereltxt = "%sysfunc(lowcase(&trRemotelyRelated))";
      else aereltxt = lowcase(aerel);
      drop aerel;
   %end;
   %else %do;
      drop aereltxt;   
   %end;
   %if %length(&AEOUT) %then %do;
      aeout = lowcase(aeout);
   %end;
   
   %if %upcase(&FFlA) = YES and %scan(%upcase(&FTypeA),1," ") = PRIOR or %scan(%upcase(&FTypeA),1," ") = BOTH %then %do;    
      if predtA > . then PredaytxtA = catx(" ", cats("(", put(predtA, &IntDateFormat..), ","),"&trDay", cats(prestdyA,")"));
   %end;
   %else %do;
      drop PredaytxtA;
   %end;
   %if %upcase(&FFlA) = YES and %scan(%upcase(&FTypeA),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeA),1," ") = BOTH %then %do; 
      if postdtA > . then PostdaytxtA = catx(" ", cats("(", put(postdtA, &IntDateFormat..), ","),"&trDay", cats(poststdyA,")"));
   %end;
   %else %do;
      drop PostdaytxtA;
   %end;
   %if %upcase(&FFlB) = YES and %scan(%upcase(&FTypeB),1," ") = PRIOR or %scan(%upcase(&FTypeB),1," ") = BOTH %then %do; 
      if predtB > . then PredaytxtB = catx(" ", cats("(", put(predtB, &IntDateFormat..), ","),"&trDay", cats(prestdyB,")"));
   %end;
   %else %do;
      drop PredaytxtB;
   %end;
   %if %upcase(&FFlB) = YES and %scan(%upcase(&FTypeB),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeB),1," ") = BOTH %then %do; 
      if postdtB > . then PostdaytxtB = catx(" ", cats("(", put(postdtB, &IntDateFormat..), ","),"&trDay", cats(poststdyB,")"));
   %end;
   %else %do;
      drop PostdaytxtB;
   %end;
   %if %upcase(&FFlC) = YES and %scan(%upcase(&FTypeC),1," ") = PRIOR or %scan(%upcase(&FTypeC),1," ") = BOTH %then %do; 
      if predtC > . then PredaytxtC = catx(" ", cats("(", put(predtC, &IntDateFormat..), ","),"&trDay", cats(prestdyC,")"));
   %end;
   %else %do;
      drop PredaytxtC;
   %end; 
   %if %upcase(&FFlC) = YES and %scan(%upcase(&FTypeC),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeC),1," ") = BOTH %then %do;
      if postdtC > . then PostdaytxtC = catx(" ", cats("(", put(postdtC, &IntDateFormat..), ","),"&trDay", cats(poststdyC,")"));
   %end;
   %else %do;
      drop PostdaytxtC;
   %end;    
run;

proc datasets nolist;
   delete aework aesubs;
quit;

%let shrinklist = invhead invheadtxt racetxt aestdaytxt aeendaytxt;
%if %length(&AESEV) %then %do;
   %let shrinklist = &shrinklist aesevtxt;
%end;
%if &doDS %then %do;
   %let shrinklist = &shrinklist dsstdaytxt;
%end;
%if &doEX %then %do;
   %let shrinklist = &shrinklist exstdaytxt;
%end;
%if %length(&AEREL) %then %do;
   %let shrinklist = &shrinklist aereltxt;
%end;
%NObsVars(eventtext);
%if &nobs %then %do; 
%SHRINKCHARS(dsn = eventtext, vars = &shrinklist);
%end;
%let NumEvents = &nobs;
   
************************************;
***** Final Data Sets **************;
************************************;

%let AENarrative_OutData_text =;
%let AENarrative_OutData_meta =;
%PrepOutDS(AENarrative_OutData_text,OutDataName_text, &Outname.narrtext);
%let includeds = &includeds &OutDataName_text..sas7bdat,;

%if %length(&ExAEData) %then %do; 
   proc sort data = ExAELib.&ExAEData out = ExAEData;
      by usubjid aeseq;
   run;

   proc sort data = eventtext out = eventtext;
      by usubjid aeseq;
   run;
   
   data eventtext;
      merge eventtext(in = ina) ExAEData (in = inb);
      by usubjid aeseq;
      if ina;
      if ina and inb then avail = 1;
      else avail = 0;
   run;
   
   proc sql noprint;         
      select sum(avail)
         into :ExMergeCheck
         from eventtext;
   quit;

   %if ^%sysfunc(strip(&ExMergeCheck)) %then %do;
      %put ERROR: None of the selected subjects and events are present in the supplemental SAS data set.;
      %put ERROR: Check if supplemental SAS data set is mismatched and either provide a new file or remove the data set from the dialog.;
      %let exiterror = 1;
      %goto exit;
%end;
      
   %if %sysfunc(findw(%upcase(&AEVARS), AESEQ)) %then %do;
      proc sort data = eventtext out = &AENarrative_OutData_text;
         by usubjid aestdtc aeterm;
   run;
%end;
%else %do;
   data &AENarrative_OutData_text;
      set eventtext;
   run;
%end;
%end;
%else %do;
   %if %sysfunc(findw(%upcase(&AEVARS), AESEQ)) %then %do;
      proc sort data = eventtext out = &AENarrative_OutData_text;
         by usubjid aestdtc aeterm;
      run;   
   %end;
   %else %do;
      data &AENarrative_OutData_text;
         set eventtext;
      run;   
   %end;
%end;

proc datasets nolist;
   delete eventtext;
quit;

%PrepOutDS(AENarrative_OutData_meta,OutDataName_meta, &Outname.narrmeta);
%let includeds = &includeds &OutDataName_meta..sas7bdat;

*** domain flags implies they are to be summarized ***;
*** assuming all variables that are required are present ***;
*** options for findings will go here ***;

data aenarrmeta;
   length CM DS EX HO MH AEACN AEACNOTH AECONTRT AEOUT AESER AEREL CMINDC ACTARM EXDAILY FDomA FDomB FDomC $ 2 TOCFl FBaseA FBaseB FBaseC FPostA FPostB FPostC FPreA FPreB FPreC CTermOnly 
          AEACNMULT AERELMULT MedHistTab ConMedTab LimitRelEvents $ 1 FLabA $ &FLabLenA FLabB $ &FLabLenB FLabC $ &FLabLenC FRangeA FRangeB FRangeC $ 9 TabPrefix $ %sysfunc(max(1,%length(&TabPrefix)))
          DOSEVAR $ 8;
   %if &doCM %then %do;
      CM = "Y";
   %end;
   %else %do;
      CM = "N";
   %end;
   %if &doDS %then %do;
      DS = "Y";
   %end;
   %else %do;
      DS = "N";
   %end;
   %if &doEX %then %do;
      EXDaily = "&EXDaily";
   %end;
   %else %do;
      EXDaily = "";
   %end;
   %if &doEX %then %do;
      EX = "Y";
   %end;
   %else %do;
      EX = "N";
   %end;    
   %if &doHO %then %do;
      HO = "Y";
      %if &HOWin = ALL %then %do;
         HOWindow = .;
      %end;
      %else %do;
         HOWindow = &HOWindow;
      %end;
   %end;
   %else %do;
      HO = "N";
      HOWindow = .;
   %end; 
   %if &doMH %then %do;
      MH = "Y";
      %if %upcase(&MedHForm) = TABLE %then %do;     
         MedHistTab = "Y";
   %end;
   %else %do;
         MedHistTab = "N";
   %end;
   %end;
   %else %do;
      MH = "N";
   %end;
   LimitRelEvents = "%substr(%upcase(&LimitRelEvents),1,1)";
   %if %length(&AEACN) %then %do;
      AEACN = "Y";
   %end;
   %else %do;
      AEACN = "N";   
   %end;
   %if %length(&AEACNOTH) %then %do;
      AEACNOTH = "Y";
   %end;
   %else %do;
      AEACNOTH = "N";   
   %end;   
   %if %length(&AECONTRT) %then %do;
      AECONTRT = "Y";
   %end;
   %else %do;
      AECONTRT = "N";   
   %end;
   %if %length(&AEOUT) %then %do;
      AEOUT = "Y";
   %end; 
   %else %do;
      AEOUT = "N";   
   %end;
   %if %length(&AEREL) %then %do;
      AEREL = "Y";
   %end; 
   %else %do;
      AEREL = "N";   
   %end;
   %if %length(&AESER) %then %do;
      AESER = "Y";
   %end;
   %else %do;
      AESER = "N";   
   %end;
   %if %length(&CMINDC) %then %do;
      CMINDC = "Y";
   %end;
   %else %do;
      CMINDC = "N";  
   %end;
   %if %length(&ACTARM) %then %do;
      ACTARM = "Y";
   %end;
   %else %do;
      ACTARM = "N";  
   %end; 
   AEWindow = &AEWindow;
   CTermOnly = "&CTermOnly";
   %if &AEACNMULT %then %do;
      AEACNMULT = "Y";
   %end;
   %else %do;
      AEACNMULT = "N";
   %end;
   %if &AERELMULT %then %do;
      AERELMULT = "Y";
   %end;
   %else %do;
      AERELMULT = "N";
   %end;
   %if &doCM %then %do;
      CMWindow = &CMWindow;
      ConMedTab = "&ConMedTab";
      %if %sysfunc(findw(%upcase( &CMVARS), CMDOSE)) %then %do;
         DOSEVAR = "CMDOSE";
   %end;
      %else %if %sysfunc(findw(%upcase( &CMVARS), CMDOSTXT)) %then %do;
         DOSEVAR = "CMDOSTXT";
      %end;
   %end;
   %else %do;
      CMWindow = .;
   %end;
   %if %upcase(&FFlA) = YES %then %do;
      FDomA = "&FFDomAU";
      FLabA = "&FLabA";
      %if &FFDomAU = LB %then %do;
         FRangeA = "%upcase(&FRangeA)";      
      %end;
      %else %do;
         drop FRangeA;
      %end;
      %if %upcase(&FBaseA) = YES  %then %do; 
         FBaseA = "Y";
      %end;    
      %else %do;
         FBaseA = "N";
      %end;
      %if %scan(%upcase(&FTypeA),1," ") = PRIOR or %scan(%upcase(&FTypeA),1," ") = BOTH %then %do; 
         FPreA = "Y";
      %end;
      %else %do;
         FPreA = "N";
      %end;
      %if %scan(%upcase(&FTypeA),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeA),1," ") = BOTH %then %do; 
         FPostA = "Y";
      %end;    
      %else %do;
         FPostA = "N";
      %end;    
   %end;
   %else %do;
      FDomA = "";
      drop FBaseA FPostA FPreA FLabA FRangeA;
   %end;
   %if %upcase(&FFlB) = YES %then %do;
      FDomB = "&FFDomBU";
      FLabB = "&FLabB";
      %if &FFDomBU = LB %then %do;
         FRangeB = "%upcase(&FRangeB)";      
      %end;
      %else %do;
         drop FRangeB;
      %end;
      %if %upcase(&FBaseB) = YES  %then %do; 
         FBaseB = "Y";
      %end;    
      %else %do;
         FBaseB = "N";
      %end;
      %if %scan(%upcase(&FTypeB),1," ") = PRIOR or %scan(%upcase(&FTypeB),1," ") = BOTH %then %do; 
         FPreB = "Y";
      %end;
      %else %do;
         FPreB = "N";
      %end;
      %if %scan(%upcase(&FTypeB),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeB),1," ") = BOTH %then %do; 
         FPostB = "Y";
      %end;    
      %else %do;
         FPostB = "N";
      %end;    
   %end;
   %else %do;
      FDomB = "";
      drop FBaseB FPostB FPreB FLabB FRangeB;
   %end;  
   %if %upcase(&FFlC) = YES %then %do;
      FDomC = "&FFDomCU";
      FLabC = "&FLabC";
      %if &FFDomCU = LB %then %do;
         FRangeC = "%upcase(&FRangeC)";      
      %end;
      %else %do;
         drop FRangeC;
      %end;    
      %if %upcase(&FBaseC) = YES  %then %do; 
         FBaseC = "Y";
      %end;    
      %else %do;
         FBaseC = "N";
      %end;
      %if %scan(%upcase(&FTypeC),1," ") = PRIOR or %scan(%upcase(&FTypeC),1," ") = BOTH %then %do; 
         FPreC = "Y";
      %end;
      %else %do;
         FPreC = "N";
      %end;
      %if %scan(%upcase(&FTypeC),1," ") = SUBSEQUENT or %scan(%upcase(&FTypeC),1," ") = BOTH %then %do; 
         FPostC = "Y";
      %end;    
      %else %do;
         FPostC = "N";
      %end;    
   %end;
   %else %do;
      FDomC = "";
      drop FBaseC FPostC FPreC FLabC FRangeC;
   %end;   
   TabStartNum = &TabStartNum;
   TabPrefix = "&TabPrefix";
   NumEvents = &NumEvents;
   TOCFl = "&TOCFl";
run;

data &AENarrative_OutData_meta;
   set aenarrmeta;
run;

proc datasets nolist;
   delete aenarrmeta;
quit;

%if ^&exiterror %then %do;
    /*
     * Create TSV files for narrative metadata and data
     */ 
    options nobom;
    %let narMetaFilename = %nrbquote(narrativeMetadata.tsv);
    %let narMetaFilePath = %nrbquote(&OutPath.&narMetaFilename);
    filename narrOutF "&narMetaFilePath" encoding="UTF-8";
    proc export data=&AENarrative_OutData_meta outfile=narrOutF dbms=tab replace; run;
    %PackageInsertFile(Metadata file for narrative from &ProcessName, &OutPath, &narMetaFilename, , , yes, no);

    %let narDataFilename = %nrbquote(narrativeTextData.tsv);
    %let narDataFilePath = %nrbquote(&OutPath.&narDataFilename);
    filename narrOutF "&narDataFilePath" encoding="UTF-8";
    proc export data=&AENarrative_OutData_text outfile=narrOutF dbms=tab replace;
    %PackageInsertFile(Data file for narrative from &ProcessName, &OutPath, &narDataFilename, , , yes, no);
    
    %let narAEFilename = %nrbquote(narrativeAEdata.tsv);
    %let narAEFilePath = %nrbquote(&OutPath.&narAEFilename);
    filename narrOutF "&narAEFilePath" encoding="UTF-8";
    proc export data=&AENarrative_OutData_event outfile=narrOutF dbms=tab replace; run;
    %PackageInsertFile(AE data file for narrative from &ProcessName, &OutPath, &narAEFilename, , , yes, no);
    
    %if %upcase(&FFlA) = YES %then %do;
       %let narDOMAFilename = %nrbquote(narrative&FFDomAU.data.tsv);
       %let narDOMAFilePath = %nrbquote(&OutPath.&narDOMAFilename);
       filename narrOutF "&narDOMAFilePath" encoding="UTF-8";
       proc export data=&AENarrative_OutData_domA outfile=narrOutF dbms=tab replace; run;
       %PackageInsertFile(&FFDomAU data file for narrative from &ProcessName, &OutPath, &narDOMAFilename, , , yes, no);                
    %end;

    %if %upcase(&FFlB) = YES %then %do;
       %let narDOMBFilename = %nrbquote(narrative&FFDomBU.data.tsv);
       %let narDOMBFilePath = %nrbquote(&OutPath.&narDOMBFilename);
       filename narrOutF "&narDOMBFilePath" encoding="UTF-8";
       proc export data=&AENarrative_OutData_domB outfile=narrOutF dbms=tab replace; run;
       %PackageInsertFile(&FFDomBU data file for narrative from &ProcessName, &OutPath, &narDOMBFilename, , , yes, no);                
    %end;

    %if %upcase(&FFlC) = YES %then %do;
       %let narDOMCFilename = %nrbquote(narrative&FFDomCU.data.tsv);
       %let narDOMCFilePath = %nrbquote(&OutPath.&narDOMCFilename);
       filename narrOutF "&narDOMCFilePath" encoding="UTF-8";
       proc export data=&AENarrative_OutData_domC outfile=narrOutF dbms=tab replace; run;
       %PackageInsertFile(&FFDomCU data file for narrative from &ProcessName, &OutPath, &narDOMCFilename, , , yes, no);                
    %end;
    
   %if &doCM %then %do;
      %let narCMFilename = %nrbquote(narrativeCMdata.tsv);
      %let narCMFilePath = %nrbquote(&OutPath.&narCMFilename);
      filename narrOutF "&narCMFilePath" encoding="UTF-8";
      proc export data=&AENarrative_OutData_CM outfile=narrOutF dbms=tab replace; run;
      %PackageInsertFile(CM data file for narrative from &ProcessName, &OutPath, &narCMFilename, , , yes, no);                
   %end;   
    
    /*
     * Create JSL and send package to client for rendering
     */
    %PrepOutFile(AENarrativeOuputFilePath,AENarrativeOuputFilename,&ProcessName._&StudyName._Output.rtf);
    %PrepOutFile(AENarrativeScript,AENarrativeScriptName,&ProcessName._&StudyName._Script.jsl);
    %global _mainjslfile_ FullAPName;
    %let _mainjslfile_ = &AENarrativeScript;
    %let FullAPName=%str(AE Narrative);
    
    %put AENarrativeScript = &AENarrativeScript;
    %put AENarrativeScriptName = &AENarrativeScriptName;
    %put AENarrativeOuputFilePath = &AENarrativeOuputFilePath;
    %put AENarrativeOuputFilename = &AENarrativeOuputFilename;
    
    data _null_;
       file "&AENarrativeScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
        put "//!";
        put "Names Default to Here(1);";
        put "objRefNS = newnamespace();";
        put "jg:currentObjRefNamespaceName = objRefNS << getName;";        
        put "objRefNS:DataList = {};";
        put "objRefNS:TabButtonList = {};";
        put "objRefNS:TabCodeList = {};";
        put "objRefNS:TabAfterCodeList = {};";
        put "objRefNS:TabOpenList = {};";
        put "objRefNS:CreateReportList = {};";
        put "jg:NarrativeTemplateDefaultName = ""&NarrativeTemplate"";";
        /* Kemicl added to accommodate change in TabbedReport */
        put "objRefNS:ShowDataTables = 0;";
        put "objRefNS:ShowCreateReport = 0;";
        put "objRefNS:ShowNotes = 0;";
        
         /* kemicl try to delete the ae narrative drill down filter */
         %if %nrbquote(%upcase(&SubjectFilter)) = AE NARRATIVE DRILLDOWN %then %do;
            put "Include(""$CLINICAL_HOME\JSLFiles\SubjectFilterUtilityFunctions.jsl"");"; 
            put "try(DeleteSubjectFilterByName(""AE Narrative Drilldown"", empty(), ""&studyname""));";
         %end;        
        put "if (!fileExists(""&ClientOutPath.&AENarrativeOuputFilename""),";
        put "    Include(ConvertFilePath(""$JMP_HOME/Clinical/ProcessLibrary/AENarrativeTemplateProcessor.jsl"", absolute, windows));";
        put "    processAENarrativeTemplate(""&ClientOutPath.&narMetaFilename"",";
        put "    ""&ClientOutPath.&narDataFilename"",";
        put "    ""&ClientOutPath.&AENarrativeOuputFilename"",";
        put "    &EVTSPRFL, ""&StudyName"");";
        put "    deleteFile(""&ClientOutPath.&narMetaFilename"");";
        put "    deleteFile(""&ClientOutPath.&narDataFilename"");";
        put "    deleteFile(""&ClientOutPath.&narAEFilename"");";
        %if %upcase(&FFlA) = YES %then %do;
            put "    deleteFile(""&ClientOutPath.&narDOMAFilename"");";
        %end;
        %if %upcase(&FFlB) = YES %then %do;
            put "    deleteFile(""&ClientOutPath.&narDOMBFilename"");";
        %end;
        %if %upcase(&FFlC) = YES %then %do;
            put "    deleteFile(""&ClientOutPath.&narDOMCFilename"");";
        %end;
        %if &doCM %then %do;
            put "    deleteFile(""&ClientOutPath.&narCMFilename"");";
        %end;
        put ");";
        %if &runAsDrillDown %then %do;
            put "objRefNS:narrativeReportFilename = ""&AENarrativeOuputFilename"";";
            put "Open(""&ClientOutPath.&AENarrativeOuputFilename"");";
        %end;
        %else %do;
        put "objRefNS:fbox=Function({},{default local},";
        put "   Vlistbox(";
        %if %length(&TEAEText) %then %do;
           put "objRefNS:TEAEText = TextBox(""&TEAEText""), Text Box(""""),";
        %end;        
        put "      objRefNS:Button=ButtonBox(""&trOpen &trAENarrative..rtf"",";
        put "          Open(""&ClientOutPath.&AENarrativeOuputFilename"");";
        put "      ),";
        %if &ConMedTabErr %then %do;
           put "          Text Box(""""),";    
           put "          Text Box(""&trConMedTabNote""),";    
        %end;
        %if &doCM %then %do;
           %if %length(&relrecpresent) and &RELRECCM = N %then %do;
              put "          Text Box(""""),";    
              put "          Text Box(""&trRelRecCMNote"", <<Set Wrap(1000)),";            
           %end;
        %end;
        put "      objRefNS:Button << Underline Style(1);";
        put "   )";
        put ");";
        put "InsertInto(objRefNS:DataList,{{""  ""}});";
        put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""AENarrative_Text"")));";
        put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox()));";
        put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
        put "InsertInto(objRefNS:CreateReportList,{{objRefNS:TEAEText}});";
        put "InsertInto(objRefNS:TabOpenList,1);";
        %end;
    run;
    %PackageInsertFile(jsl code from &ProcessName, &OutPath, &AENarrativeScriptName, , yes);
%end;

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend AENARR;
%AENARR;

************************************;
************************************;
************************************;
