
/***************************************************************
JMP Clinical Macro

Name: Domain Distribution
Support: Kelci Miclaus, SAS Institute Inc.

Description: 
      Macro for analysis processing for AE, Events and Interventions distribution reports. 

-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
****************************************************************/


/*
$CVS_START$
$ID: ADSLDistributionOptions$
$CMP: OOUT$ $UE: DM.AGE$ $NOTES: Variable will be available in Data Filter and Distributions.$
$CMP: OOUT$ $UE: DM.SEX$ $NOTES: Variable will be available in Demographic Grouping, Data Filter, and Distributions.$
$CMP: OOUT$ $UE: DM.RACE$ $NOTES: Variable will be available in Demographic Grouping, Data Filter, and Distributions.$
$CMP: OOUT$ $UE: DM.SITEID$ $NOTES: Variable will be available in Demographic Grouping, Data Filter, and Distributions.$
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Variable will be available in Demographic Grouping, Data Filter, and Distributions.$
$CMP: OOUT$ $UE: DM.STUDYID$ $NOTES: Use like other demography only if multiple levels deteced indicating a multi-study analysis.$
$CVS_STOP$
*/

%macro DomainDistribution(domain,fillmissingVars=,countplotVars=,stackVars=,distributionVars=,customFilterVars = ,doTreatmentRRs = 0,UseGroups = 0);
   /*Create Translation Labels*/
   %let tclabel = %nrbquote(%LocaleGet(TotalCount_Text));
   %let gclabel = %nrbquote(%LocaleGet(GroupCount_Text));
   %let polabel = %nrbquote(%LocaleGet(PercentOccurrence_Text));
   %let trNone = %nrbquote(%LocaleGet(None_Text));
   %let trMissing = %nrbquote(%LocaleGet(Missing_Text));
      
   %if %length(&&in_&domain)=0 %then %do;
      %put [MS_S_0013];
      %put ERROR: The &domain or ad&domain data set does not exist for this study.;
      %let exiterror=1;
      %goto mexit;
   %end;
   
    %if %upcase(&TrtUse) = NONE %then %do;
       %let TrtName = ;
       %let TrtLabel = ;
    %end;

   %PrepOutDS(Distribution_OutData,Distribution_OutDataName,&StudyName,_&domain.d);

   /*kemicl add to attach domain to setting */
   %global OutName;
   %let OutName = &StudyName._&domain;
   %if &exiterror %then %goto mexit;   
   
   /* Check for Crossover Periods */
   %let CODateVar = &domain.STDTC;   
   %if (%sysfunc(findw(&&&domain.VARS, %upcase(&CODateVar)))=0 or %upcase(&TrtUse)=NONE or %upcase(&domain)=MH) %then %let NTrtPeriods = 1;
   /* assign crossover treatments */
   %if &NTrtPeriods > 1 %then %do;
      %AssignCOTreatment(&domain, &domain, &CODateVar);
   %end;
   %else %do;
      %let TrtVarList = ;
   %end;
   
	/* add trtvar to Byvar list */
	%if %length(&TrtName) %then %do;
		%if ^%index(&DesignByVars,&TrtName) %then %do;
			%let DesignByVars = &TrtName &DesignByVars;
		%end;
	%end;
	%if ^%symexist(SLDOSEVar) %then %let SLDOSEVar = 0;

   /* kemicl fill missing values of the Term Level and Group Level variables */
   %if %length(&fillmissingVars) %then %do;
   	%let i = 1;
      data &domain; 
         set &domain;
			%do %while (%length(%scan(&fillmissingVars,&i))>0);
				if %scan(&fillmissingVars,&i) = "" then %scan(&fillmissingVars,&i) = "&trMissing";
				%let i = %eval(&i+1);
			%end;
      run;
   %end;
   
   /* Compute Denominators for Percents from ADSL in work */
   %let adata = &in_adsl;
   %if &NtrtPeriods > 1 %then %do;
      data adsls; 
         set &adata;
         array trts{*} &TrtVarList;
         do i=1 to dim(trts);
            &TrtName = trts[i];
            output;
         end;
         drop i;
         /* Kemicl added to remove dropouts that were not in all treatment periods */
         if missing(&TrtName) then delete;
      run;
      %let adata = adsls;
   %end;
   
   /* Check number of subjects in adsl */
   %nobsvars(&adata);
   %let nsubjects = &nobs;
   %put nsubjects = &nsubjects;
   %DMFlags;
   %if &exiterror %then %goto mexit;
   
   /* Check for more than one level of Study ID to see if combined studies  */
   %let StudyLevs = ;
   %let StudyList = ;
   %if %sysfunc(findw(%upcase(&ADSLvars), STUDYID)) %then %do;
      %GetLevels(&adata,STUDYID,StudyList,_LPrefix = NLGrp,_nLvls=StudyLevs,_vType= nLGType);
   %end;

   %let DMlist = ;
   %if %length(&TrtName) %then %let DMlist = &DMList &TrtName;
   %if &doSex and %upcase(SEX) ne %upcase(&TrtName) %then %let DMlist = &DMlist SEX;
   %if &doRace and %upcase(RACE) ne %upcase(&TrtName) %then %let DMlist = &DMlist RACE;
   %if &doCtry and %upcase(COUNTRY) ne %upcase(&TrtName) %then %let DMlist = &DMlist COUNTRY;
   %if &doSite and %upcase(SITEID) ne %upcase(&TrtName) %then %let DMlist = &DMlist SITEID;
   %if (&StudyLevs > 1 and %upcase(STUDYID) ne %upcase(&TrtName) and ^%index(%Upcase(&DesignByVars),STUDYID)) %then %let DMlist = &DMlist STUDYID;
   
   %if &NtrtPeriods > 1 %then %do;
   	%let DropList = %sysfunc(tranwrd(&DMList,&TrtName,%str()));
   %end;
   %else %do;
   	%let DropList = &DMList;
   %end;
   
   data dm1; 
	  set &adata;
	  keep USUBJID &TrtName &DMlist &DesignByVars;
   run;
   
   /* IF CROSSOVER ANALYSIS: Denominator for treatment based on sum of unique subjects per treatment period */
   %let i = 1;
   
   proc sql noprint;
      %do %while (%length(%scan(&DMlist,&i))>0);
         %let j = %eval(&i+1);
         create table dm&j as select *, %if &i=1 %then 1/&nsubjects as p_None, &nsubjects as tot_None,; 1/freq(%scan(&DMlist,&i)) as p_%scan(&DMlist,&i), freq(%scan(&DMlist,&i)) as tot_%scan(&DMlist,&i)
         from dm&i
         group by %scan(&DMlist,&i)
         order by usubjid;
         %let i = &j;
      %end;
   quit;
   
   proc datasets nolist;;
      change dm&i = dm_filter;
   quit; 
   
   %if &NtrtPeriods > 1 %then %do;
      proc sort data= dm_filter;
         by USUBJID TRT;
      run;
      
      proc sort data = &domain;
         by USUBJID TRT;
      run;
   %end;
   
	%let trsdosox = %nrbquote(%LocaleGet(StudyDayofStartofX_Text));
	%let trCount = %nrbquote(%LocaleGet(Count_Text));
	%let SeverityVar = %upcase(&domain)SEV;
	%let ToxGrVar = %upcase(&domain)TOXGR;
	%let trMild = %nrbquote(%LocaleGet(Mild_Text));
	%let trModerate = %nrbquote(%LocaleGet(Moderate_Text));
	%let trSevere = %nrbquote(%LocaleGet(Severe_Text));
	%let trLifeThreatening = %nrbquote(%LocaleGet(LifeThreatening_Text));
	%let trDeath = %nrbquote(%LocaleGet(Death_Text));
	%let SeriousVar = %upcase(&domain)SER;
	%let trN = %nrbquote(%LocaleGet(N_Text));
	%let trY = %nrbquote(%LocaleGet(Y_Text));

	data domdata;
		merge &domain(in=dmf)  dm_filter ;
		by USUBJID %if &NtrtPeriods > 1 %then TRT;;
		_Count_ = 1;
		label _Count_ = "&trCount";
		%if %upcase(&domain) = SU %then %do;
			%let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(SubstanceAbuse_Text)))); 
			if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
				label &domain.STDY = "&trlabel";
			end;
		%end;
		%else %if %upcase(&domain) = MH %then %do;
			%let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(MedicalHistoryEvent_Text))));
			label &domain.STDY = "&trlabel";
		%end;       
		%else %if %upcase(&domain) = DS %then %do;
			%let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(DispositionEvent_Text))));
			if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
				label &domain.STDY = "&trlabel";
			end;
		%end;       
		%else %if %upcase(&domain) = CE %then %do;
			%let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(ClinicalEvent_Text))));
			if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
				label &domain.STDY = "&trlabel";
			end;
		%end;
		%else %if %upcase(&domain) = DV %then %do;
			%let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(ProtocolDeviation_Text))));
			if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
				label &domain.STDY = "&trlabel";
			end;
		%end;                
		%else %do;
			if missing(&TmName) then delete;
		%end;
		%if %sysfunc(findw(&&&domain.VARS, &SeriousVar)) %then %do;
			if upcase(&SeriousVar)="N" or kupcase(&SeriousVar)=kupcase("&trN") then SERIOUS = 1;
			else if upcase(&SeriousVar)="Y" or kupcase(&SeriousVar)=kupcase("&trY") then SERIOUS = 2;
         else SERIOUS = 0;
		%end;
		%if %sysfunc(findw(&&&domain.VARS, &SeverityVar)) %then %do;
			if upcase(&SeverityVar)="MILD" or kupcase(&SeverityVar)=kupcase("&trMild") then SEVERITY = 1;
			else if upcase(&SeverityVar)="MODERATE" or kupcase(&SeverityVar)=kupcase("&trModerate") then SEVERITY = 2;
			else if upcase(&SeverityVar)="SEVERE" or kupcase(&SeverityVar)=kupcase("&trSevere") then SEVERITY = 3;
			else if upcase(&SeverityVar)="LIFE THREATENING" or kupcase(&SeverityVar)=kupcase("&trLifeThreatening") then SEVERITY = 4;
			else if upcase(&SeverityVar)="DEATH" or kupcase(&SeverityVar)=kupcase("&trDeath") then SEVERITY = 5;
			else SEVERITY = 0;
	   %end;
		if dmf;
	run;;
   
   %nobsvars(domdata);
   %if &nobs=0 %then %do;
      %put [MS_S_0208];
      %put ERROR: The merge of the %upcase(&domain) and &in_adsl data sets resulted in 0 observations. Please check your data for accuracy.;
      %let exiterror=1;
      %goto mexit;
   %end;
   
   %let domainlabellist = ;
   %if %length(&stackVars) %then %do;
      %let i = 1;
      %do %while (%length(%scan(&stackVars,&i)));
         %let currvar = %scan(&stackVars,&i);
         %if %length(&currvar) %then %do;
            %let &currvar.Label = &currvar;
            %CheckLabel(domdata, &currvar.Label);
            %let domainlabellist = &domainLabelList &&currvar.Label;
         %end;
         %let i = %eval(&i+1);
      %end;
   %end;
   
   /* KELCI:  NEED TO HANDLE COUNTS OF EVENTS WITH THE SAME VALUE BUT DIFFERENT GROUP */
   /* create a first occurrence flag in the output data set based on sort date */
   /* kemicl account for crossover to count unique events per subject per period */
   %if %upcase(&CountMultiple)=NO %then %do;
      proc sort data=domdata out=domdata;
            by %if &UseGroups %then &TmGroup; &TmName  %if &NTrtPeriods > 1  %then PERIOD; USUBJID 
               %if  %sysfunc(findw(&&&domain.VARS, &SeriousVar)) %then %do; 
               	descending SERIOUS 
               %end; 
               %if %sysfunc(findw(&&&domain.VARS, &SeverityVar)) %then %do; 
               	descending SEVERITY
               %end; 
               %if %sysfunc(findw(&&&domain.VARS, &ToxGrVar)) %then %do; 
               	descending &ToxGrVar
               %end;                
               &domain.STDY;
      run;
      
      data domdata; set domdata;
         by %if &UseGroups %then &TmGroup; &TmName %if &NTrtPeriods > 1 %then PERIOD; USUBJID;
         if first.USUBJID then output;
      run;
   %end;
   
	/*create counts for barcharts, used to be treemaps*/
	/* create counts for ordering */
	/* kemicl written to generically hand making more count values but first value of list MUST be TmName */
	%let createcountvars = &countplotVars;
	%if %length(&TmGroup) %then %do;
		%let createcountvars = &countplotVars &TmGroup;
	%end;

	%let i = 1;
	%do %while(%length(%scan(&createcountvars,&i)));
		%let percentvar = %scan(&createcountvars,&i);

		proc sort data=domdata(keep= %if &UseGroups and &i = 1 %then &TmGroup; &percentvar) out=getcounts;
			%if &UseGroups and &i = 1 %then %do;
				by &TmGroup &percentvar;
			%end;
			%else %do;
				by &percentvar;
			%end;
		run;

		proc freq data=getcounts noprint;
			%if &UseGroups and &i = 1 %then %do;
				tables &TmGroup*&percentvar / out=countdt;
			%end;
			%else %do;
				tables &percentvar / out=countdt;
			%end;
		run;

		%if %upcase(&CountMultiple)=NO and &i=1  %then %do;
			data countdt; set countdt;
				Percent = round((count/&nsubjects)*100,0.1);
				label Percent = "&polabel";
			run;
		%end;

		proc datasets;
			modify countdt;
			%if &i = 1 %then %do;
				label count = "&tclabel";
			%end;
			%else %do;
				rename count = group_count;
				label group_count = "&gclabel";
			%end;
		quit;

		proc sql;
			create table domdata as
			select * from domdata as d1 left join countdt as d2 
			on %if &UseGroups and &i = 1 %then d1.&TmGroup=d2.&TmGroup and; d1.&percentvar=d2.&percentvar
			order by USUBJID;
		quit;
		%let i = %eval(&i+1);
	%end; 

	%let GroupVarLabel = ;
	%if %symexist(TmGroup) %then %do;
		%if %length(&TmGroup) %then %do;
			%let GroupVarLabel = &TmGroup;
			%CheckLabel(domdata, GroupVarLabel);
		%end;
	%end;
	%else %let TmGroup = ;
	
   %DMLabels(&adata);

   /* Treatment Incidence Filter, for now group across crossovers*/
   %if &doTreatmentRRs %then %do;
      proc sort data=domdata(keep = &TmName &TrtName p_&TrtName) out=ddsort;
         by &TmName &TrtName; 
      run;
      
      proc means data=ddsort;
         by &TmName &TrtName; 
         var p_&TrtName;
         output out=ddsums sum(p_&TrtName) = TrtIncidence;
      run;
      
      proc transpose data = ddsums out=sumst(drop=_name_);
         by  &TmName;
         var TrtIncidence;
         id &TrtName;
         idlabel &TrtName;
      run;
      
      %sasname(&&tlev&plev);
      %let sascontrollev = &sasname;
      %do i = 1 %to %eval(&nlev);
         %sasname(&&tlev&i);
         %let sastrtlev&i = &sasname;
         %let rrlabel&i = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(RelativeRiskforX_Text)), f1 = %nrbquote(&&tlev&i)));
      %end;
      %let maxrrlabel = %nrbquote(%LocaleGet(MaxTreatmentRR_Text));
      %let trtocclabel = %nrbquote(%LocaleGet(TreatmentOnlyOccurrenceFlag_Text)); 
      
      data rrs(keep = &TmName RR: %if %eval(&nlev) > 2 %then MaxRRValue; TrtoccFl);
         set sumst;
         %do i = 1 %to %eval(&nlev);
            %if &i ^= &plev %then %do;
               RR_trtlev&i._over_control = round(&&sastrtlev&i/&sascontrollev,0.01);
               label RR_trtlev&i._over_control = &&rrlabel&i;
            %end;
         %end;
         %if %eval(&nlev) > 2 %then %do;
            MaxRRvalue = max(of rr:);
            label MaxRRvalue = &maxrrlabel;
         %end;
         if missing(&sascontrollev) then TrtOccFl = 1;
         else TrtOccFl = 0;
         label TrtOccFl= "Treatment Only Occurrence Flag";
      run;
      
      data domdata;
         set domdata;
         if 0 then set rrs;
         if _n_ = 1 then do;
            declare hash h(dataset:"rrs");
            h.defineKey("&TmName");
            h.defineData(ALL:"YES");
            h.defineDone();
         end;
         
         if h.find() = 0;
      run;
   %end; 
   
   %if ^%symexist(PropThresh) %then %let PropThresh = 0;
	/*create final output data */
	data &Distribution_OutData; 
		set domdata; 
		%if &PropThresh > 0 %then %do;
			where Percent >= &PropThresh;
		%end;
		drop &DropList; 
   run;   
  
   
	%nobsvars(&Distribution_OutData);
	%if &nobs = 0 %then %do;
		%put ERROR: There are no observations in the analysis data set after applying report filters and processing data.;
		%let exiterror=1;
		%goto mexit;
	%end;
	
	/* GET Variable Labels */
   %let SubjectLabel = USUBJID;
   %let DecodLabel = &TmName;
   %let StdyLabel = &domain.STDY;
   %CheckLabel(domdata,SubjectLabel);   
   %CheckLabel(domdata,StdyLabel);
   %CheckLabel(domdata,DecodLabel);
   %let doEPOCH = 0;
   %let doEPOCH =  %sysfunc(findw(&&&domain.VARS,EPOCH));
   %if &doEPOCH %then %do;
      %let EpochLabel = EPOCH;
      %CheckLabel(domdata,EpochLabel);
   %end;
   %if %length(&distributionVars) %then %do;
      %let i=1;
      %let f=1;
      %do %while(%length(%scan(&distributionVars,&i)));
         %let ev = %scan(&distributionVars,&i); 
         %let &ev.Label = %upcase(&ev);
         %let vnum = %sysfunc(findw(&&&domain.VARS,&&&ev.Label));
         %if &vnum > 0 %then %do;
            %CheckLabel(domdata,&ev.Label);  
            %let evl&f = &&&ev.Label;
            %let f=%eval(&f+1);
         %end;
         %let i=%eval(&i+1);
      %end;
      %let nev = %eval(&f-1);
   %end;    
   %else %let nev = 0;

	%if %length(&customFilterVars) %then %do;
      %let i=1;
      %let f=1;
      %do %while(%length(%scan(&customFilterVars,&i)));
         %let dfv = %scan(&customFilterVars,&i); 
         %let &dfv.Label = %upcase(&dfv);
         %let vnum = %sysfunc(findw(&&&domain.VARS,&&&dfv.Label));
         %if &vnum > 0 %then %do;
            %CheckLabel(domdata,&dfv.Label);  
            %let dfvl&f = &&&dfv.Label;
            %let f=%eval(&f+1);
         %end;
         %let i=%eval(&i+1);
      %end;
      %let ndfv = %eval(&f-1);
   %end;    
   %else %let ndfv = 0;

   *** Added 2 to label so TRTLABEL from CMTRT doesnt overwrite treatment label ***;
   %if %length(&countplotVars) %then %do;
      %let i=1;
      %let f=1;
      %do %while(%length(%scan(&countplotVars,&i)));
         %let tv = %scan(&countplotVars,&i);
         %let &tv.Label2 = %upcase(&tv);
         %let vnum = %sysfunc(findw(&&&domain.VARS,&&&tv.Label2));
         %if &vnum > 0 %then %do;
            %CheckLabel(domdata,&tv.Label2); 
            %let tvl&f = &&&tv.Label2;
            %let f=%eval(&f+1);
         %end;
         %let i=%eval(&i+1);
      %end;
      %let ntv = %eval(&f-1);
   %end;    
   %else %let ntv = 0;  

   %let odata = domdata;
   %include "&MacroPath./DesignByVars.sas";
   
   %if %upcase(&CountMultiple)=NO %then %let AxLab = %nrbquote(%LocaleGet(Subjects_Text));
   %else %let AxLab = %nrbquote(%LocaleGet(Occurrences_Text));
   
   /*Set up macros for cross drilldowns*/
   /*
   %if %upcase(&domain) = CM %then %do;
      %let APddname = AEDistribution;
      %let crossdomain = AE;
   %end;
   %else %if %upcase(&domain) = AE %then %do;
      %let APddname = InterventionsDistribution;
      %let crossdomain = CM;
   %end;
   */
   
   * create JSL file;
   %PrepOutFile(DistributionScript,DistributionScriptName,&StudyName &ProcessName Script for &domain..jsl);
   %global _mainjslfile_ FullAPName;
   %let _mainjslfile_ = &DistributionScript;
   %let FullAPName=%str(%upcase(&domain) Distribution);
   
   %let DeltaLabel = ;  
   * For previous added study, there is no global variable for ReviewsEnabled; 
   %if ^%symexist(ReviewsEnabled) %then %do;
      %global ReviewsEnabled;
      %let ReviewsEnabled=1;
   %end;
   %if &ReviewsEnabled  and &ReviewNumber %then %do;
      %let DeltaLabel = &domain.RFlg;
      %CheckLabel(domdata,DeltaLabel);
   %end;
   
   %if &StudyLevs > 1 %then %do;
      %put Data contains records from &StudyLevs Studies;
      %let StudyLabel = STUDYID;
      %CheckLabel(&adata, StudyLabel);
   %end;
   
   * put JSL statements;
   data _null_;
      file "&DistributionScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newNamespace();";
      put "jg:currentObjRefNamespaceName = objRefNS << getName;";
      put "objRefNS:DataList = {};";
      put "objRefNS:DataNameList = {};";
      put "objRefNS:TabButtonList = {};";
      put "objRefNS:TabCodeList = {};";
      put "objRefNS:TabAfterCodeList = {};";
      put "objRefNS:TabOpenList = {};";
      put "objRefNS:CreateReportList = {};";
      put "objRefNS:TMView = 0;";
      put "objRefNS:Percents = 0;";
      put "objRefNS:NoteDomain = ""%upcase(&domain)"";";
      /*put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
      put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";*/

      
      /* Try to Delete Domain Distribution Filter */
      %if %symexist(SFname) %then %do;
         %if %upcase(&SFname) = DISTRIBUTION_DRILL_DOWN %then %do;
            put "Include(""$CLINICAL_HOME\JSLFiles\SubjectFilterUtilityFunctions.jsl"");"; 
            put "try(DeleteSubjectFilterByName(""Distribution Drill Down"", empty(), ""&studyname""));";
         %end;
      %end;
      /*%if %upcase(&ProcessName) = EVENTSDISTRIBUTION %then %do;
         put " objRefNS:includestring= ""Events: "";"; 
      %end;
      %else %if %upcase(&ProcessName) = AEDISTRIBUTION %then %do;
         put " objRefNS:includestring=""Adverse Events: "";";
      %end;
      %else %do;
         put " objRefNS:includestring=""Interventions: "";";
      %end;*/
      /*put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&DomainWhereStmt)]\"");";
      
      %if %length(&WhereForReports) %then %do;
         put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
         put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
         %if %length(&DomainWhereStmt) %then %do;
            put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
         %end;
      %end;
      %else %if %length(&DomainWhereStmt) %then %do;
         put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;*/    

      put "OutPath= ""&OutPath"";";
      /*put "objRefNS:PopulationSet= ""&PopulationSet"";";*/
      put "ClientOutPath=""&ClientOutPath"";";
      put "outFolder = ""&ClientOutPath"";";
      
      put "DtExpr  = Expr(";
      put "   If(IsEmpty(objRefNS:dmd),";
      put "      objRefNS:dmd=open(""&ClientOutPath.&Distribution_OutDataName..sas7bdat"",invisible);";
      put "      reportRefNS:linkAnalysisTableToADSLTable(objRefNS:dmd);";
      put "      if (reportRefNS:reviewReplayIncludeDataPreprocessing,";
      put "      	  jg:ApplyValueOrderToTables(EvalList({objRefNS:dmd}), ""&StudyName"");";      
      put "         Column(""&SubjectLabel"") << Label(1);";  
      %if %length(&StdyLabel) %then %do;
         put "       Try(Column(""&StdyLabel"") << Set Modeling Type( Continuous));";
      %end; 
		put "        New Column(""%  "",";
		%if %length(&TrtLabel) %then %do;
			put "       set Formula((:Name(""p_&TrtName"")))";
		%end;
		%else %do;
			put "       set Formula((:Name(""p_None"")))";
		%end;
		put "       );";

		put "        New Column(locale:get(""N_Text""),";
		%if %length(&TrtLabel) %then %do;
			put "       set Formula((:Name(""tot_&TrtName"")))";
		%end;
		%else %do;
			put "       set Formula((:Name(""tot_None"")))";
		%end;
		put "       );"; 
      %if %length(&GroupVarLabel) %then %do;
         put "    objRefNS:dmd << Color by Column(:Name(""&GroupVarLabel""), Color Theme( ""JMP Light"" ));";
         put "    vcoloraa = Associative Array();";
         put "    For Each Row( vcoloraa[Column(""&GroupVarLabel"")[Row()]] = Color Of( Row State( Row())));";
         put "    vcolors = vcoloraa << Get Values;";
         put "    For(";
         put "       v = 1;";
         put "       vc = vcoloraa <<first;, !IsEmpty(vc), vc = vcoloraa <<next(vc);";
         put "       v++;,";
         put "       vcolors[v] = Substitute(Expr(_label_ = _color_), Expr( _label_ ), vc, Expr( _color_ ), vcoloraa[vc])";
         put "    );";
         put "    Column(""&GroupVarLabel"") << Set Property( ""Value Colors"", vcolors);";
      %end;
      %if &doTreatmentRRs %then %do;
         put "    Try(Column(""&trtocclabel"") << Set Modeling Type(Nominal));";
      %end;
      put "		,";  /* else for reviewReplayInclude */
		%if %length(&TrtLabel) %then %do;
			put "      Column(""%  "")<<set Formula((:Name(""p_&TrtName"")));";
			put "      Column(locale:get(""N_Text""))<<set Formula((:Name(""tot_&TrtName"")))";
		%end;
		%else %do;
			put "       Column(""%  "")<<set Formula((:Name(""p_None"")));";
			put "       Column(locale:get(""N_Text""))<<set Formula((:Name(""tot_None"")))";	
		%end;  
      put "       ); //End of check for reportRefNS:reviewReplayIncludeDataPreprocessing";
      put "  ,";  /*else for isempty*/;
      put "    Current Data Table(objRefNS:dmd);";
      put "  );";
      put "objRefNS:dbv = {};";
      %do i=1 %to &ndbv;
         %if %quote(&&dbv&i) ne %quote(&TrtLabel) %then %do;
            put "InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
         %end;
      %end;
      put ");";
      /* run/open data table now before results function calls */
      put "DtExpr;"; 
      
      %if %upcase(&domain) = DV %then %let yaxis = %eval(10*&TMLen);
      %else %let yaxis = 625;
      
      %macro barplot1(ii,groupx1,overx2);
         put "    VListBox(";
         put "    objRefNS:gb&ii = Graph Builder(";
         put "       Size(675,&yaxis),";

         /*%if %upcase(&SplitResults)=YES %then %do;
            put "         By(eval(objRefNS:dbv)),";
         %end;    
         */
         put "       Show Control Panel(0), Show Legend(1), Auto Stretching(0),";
         put "       Variables(";
         %if &UseGroups %then %do;
         	put "			X( :Name(""&GroupVarLabel""), Order By(:Name(""&gclabel""), Descending)),";
         %end;
         put "          X( :Name(""&&tvl&ii"")";
         put "             ,Position(1)";
         put "            ,Order By(:Name(""&tclabel""), Descending)";
         put "          )";
         %do b=2 %to &ndbv;
            put "    , Group X( :Name(""&&dbv&b"") )";
         %end;
         %if %length(&groupx1) %then %do;
            put "    ,Group Y( Column(&groupx1) )";
         %end;       
         %if %length(&overx2) %then %do;
               put "    , Overlay(Column(&overx2))";
         %end;
         put "       ),";
         put "       Elements( Bar( ";
         %if &UseGroups %then %do;
         	put "			X(2),X(1), ";
         %end;
         %else %do;
         	put "			X, ";
		 %end;
         %if ^%symexist(DOMAINLABEL_&domain) %then %let DOMAINLABEL_&domain = %nrbquote(&domain);
         put "			Legend( 2 ), Bar Style( ""Stacked"" ), Summary Statistic( ""Sum"" ) ))";
         put "          ,SendToReport(";
         put "             Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),"; 
         put "             Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( locale:fill(locale:get(""XCountsGraph_Text""),";
         put " {""&&DOMAINLABEL_&domain""}) )} )";   
         put "          )";
         put "    )),";
      %mend;
 
       
 		put "objRefNS:findVarByRole= function({varlist, rolename}, {i},";
       put "	 for (i = 1, i <= n items(varlist), i++,"; 
       put "       if (varlist[i][""Role""] == rolename, Return(i));"; 
       put " );"; 
       put " 0;     // not found";
		put ");";  
		
		%macro tabulate1(ii,gpcols,firstgroup);
			%let trTotal = %nrbquote(%LocaleGet(Total_Text));
			%let Ntext = %nrbquote(%LocaleGet(N_Text));
			put "    objRefNS:tbhlb&ii = HListBox(";
			put "    objRefNS:tb1 = Tabulate(Show Control Panel( 0 ),";
			put "       Change Item Label( Statistics( N, ""&trTotal"" ) ";
			%if %upcase(&CountMultiple)=NO %then %do;
				put "       ,Statistics( Sum, ""  "" )";
			%end;
			put "       ),";
			%if %upcase(&CountMultiple)=NO %then %do;
				put "       Set Format( Sum(:Name(""%  "")(9,1,Percent))),";
			%end;
			%if %upcase(&SplitResults)=YES %then %do;
				put "         By(eval(objRefNS:dbv)),";
			%end;
			/*put "       Order by count of grouping columns( 1 ),";*/
			%if %length(&GroupVarLabel) %then %do;
				put "       Add Table( Row Table( Grouping Columns( :Name(""&GroupVarLabel""),:Name(""&&tvl&ii"") ),";
				put "       )";
			%end;
			%else %do;
				put "       Add Table( Row Table( Grouping Columns( :Name(""&&tvl&ii"") ),"; 
				put "       )";
			%end;          
			%if %length(&gpcols) %then %do;
				put "       ,Column Table( Grouping Columns( &gpcols ) ";
				%if %upcase(&CountMultiple)=NO %then %do;
					put "          ,Statistics( Sum ),";
					put "          Analysis Columns( :&trCount, :Name(""%  ""))";
				%end;
				put "       )";
			%end;
			put "       ,Column Table(Statistics(N)),";
			put "    ),";  /* close Add Table */
			%if %length(&GroupVarLabel) %then %do;
				put "    Display Column Width( Row Label(""&GroupVarLabel""), 270),";
			%end;
			put "       Display Column Width( Row Label(""&&tvl&ii""), 255)";
			put " ),";  /* close tabulate */
			put " objRefNS:Table2 = Tabulate(Show Control Panel(0), Automatic Recalc(0),";
			put "    Change Item Label( Statistics( Mean, "" "" ) ),";
			put "    Add Table(";
			put "       Column Table(Analysis Columns(:Name(""&Ntext"")), Statistics(Mean))";
			%if %length(&firstgroup) %then %do;
				put "       ,Row Table( Grouping Columns(&firstgroup))";
			%end;
			put "    ),";
			put "    SendToReport(Dispatch({},""Tabulate"",";
			put "       OutlineBox,{Set Title( locale:get(""DemographyTotalSubjectCounts_Text""))})";
			put "    )";
			PUT " ),"; /*CLOSE TABULATE */
			put " ),"; /* close Hlistbox */
		%mend;
      
      put "objRefNS:ddw=Function({},{default local},";
      put " DtExpr;";
      %if (&ntv) %then %do;
			put "objRefNS:CSlist = {};";
			%if &doSite %then %do;
				put "InsertInto(objRefNS:CSlist,Column(""&SiteLabel""));";
			%end;
			%if &doCtry %then %do;
				put "InsertInto(objRefNS:CSlist,Column(""&CtryLabel""));";
			%end;
			%if &doSex %then %do;
				put "InsertInto(objRefNS:CSlist,Column(""&SexLabel""));";
			%end;
			%if &doRace %then %do;
				put "InsertInto(objRefNS:CSlist,Column(""&RaceLabel""));";
			%end;

			%do i=1 %to &ndbv;
				%if %quote(&&dbv&i) ne %quote(&TrtLabel) %then %do;
					put "InsertInto(objRefNS:CSlist,Column(""&&dbv&i""));";
				%end;
			%end;      
         put "       objRefNS:tmlist={";
         put "          ""&trNone"" ";       
         %if %length(&TrtLabel) %then %do;
            put "       ,""&TrtLabel"" ";
         %end;
         %if &doSex %then %do;
            put "       , ""&SexLabel"" ";
         %end;
         %if &doRace %then %do;
            put "       , ""&RaceLabel"" ";
         %end;                   
         %if &doCtry %then %do;
            put "       , ""&CtryLabel"" ";
         %end;    
         %if &doSite %then %do;
            put "       , ""&SiteLabel"" ";
         %end;
         %if ^%index(%Upcase(&DesignByVars),STUDYID) and &StudyLevs > 1 %then %do;
            put "       , ""&StudyLabel"" ";
         %end;
         put "       };";        
         
         put "       objRefNS:tm2list={";
         put "          ""&trNone"" ";
         %if &NtrtPeriods > 1 %then %do;
            put "       ,  ""Period"" ";
         %end;
         /* kemicl Process any stackvars present to put int widget*/
			%let j = 1;
			%do %while (%length(%scan(&domainLabelList,&j)));
				%let curravar = &%scan(&domainLabelList,&j);
				put "       ,""&curravar"" ";
				%let j = %eval(&j+1);
			%end;
         %if %upcase(&domain) = EX %then %do;
            %if &SLDOSEVar %then %do;
               put "    , ""&DoseLabel"" ";
            %end;
            %if &SLCATVar %then %do;
               put "    ,  ""&CatLabel"" ";
            %end;
         %end;
         %if &doEPOCH %then %do;
            put "       , ""&EPOCHLabel"" ";
         %end; 
         %if %length(&DeltaLabel) %then %do;
            put "          , ""&DeltaLabel"" ";
         %end;
         put "       };";
         /*
         put "   for(t=2,t<=NItems(objRefNS:tmlist),t++,";
         put "      cts = (CurrentDataTable() << Summary(Group(:(objRefNS:tmlist[t])), N)) << Sort(By(Column(3)),Order(Descending),Replace Table);";
         put "      valueorder = Column(objRefNS:tmlist[t])<<GetAsMatrix;";
         put "      Close(cts,nosave);";
         put "     :(objRefNS:tmlist[t]) << Set Property(""Value Ordering"",valueorder);";
         put "   );";  
         */
         /*Drew added the extra call to DtExpr to force reset of current data table*/
         put "   DtExpr;";
         put "   jg:ApplyValueOrderToTables(EvalList({CurrentDataTable()}), ""&StudyName"");";         
         /* kemicl Create associative array for percents */
         put "objRefNS:plist = {""p_None""};";
         put "objRefNS:nlist = {""tot_None""};";
         %let k = 1;
         %do %while (%length(%scan(&DMlist,&k))>0);
            put "InsertInto(objRefNS:plist,""p_%scan(&DMlist,&k)"");";
            put "InsertInto(objRefNS:nlist,""tot_%scan(&DMlist,&k)"");";
            
            %let k = %eval(&k+1);
         %end;
         put " objRefNS:CParray = Associative Array(objRefNS:tmlist,objRefNS:plist);";
         put " objRefNS:CNarray = Associative Array(objRefNS:tmlist,objRefNS:nlist);";
         
         put "    objRefNS:GBPexpr = expr(";
         %do i = 1 %to &ntv;        
            put "          objRefNS:gbr&i = report(objRefNS:gb&i)[Graph Builder Box(1)];";
            put "          objRefNS:gbr&i << Add Variable({:Name(""%  ""), Role(""Y"")});";
            put "          objRefNS:gb&i << SendToReport(";
            put "             Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
            put "             Dispatch({},""Y title"", TextEditBox,";
            put "                {Rotate Text(""Left""), Set Text(locale:get(""PercentOccurrenceinDemographicGroup_Text""))}";
            put "          ));";
            put "          report(objRefNS:gb&i)[AxisBox(2)]<<format(""Percent"",9,0);";
         %end;
         put "    );";
         
         put "  objRefNS:GBCexpr = expr(";
         %do i = 1 %to &ntv;
            put "          objRefNS:gb&i << SendToReport(";
            put "             Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
            put "             Dispatch({},""Y title"", TextEditBox,";
            %if %upcase(&CountMultiple)=YES %then %do;
               put "                {Rotate Text(""Left""), Set Text(locale:get(""Count_Text""))}";
            %end;
            %else %do;
               put "                {Rotate Text(""Left""), Set Text(locale:get(""SubjectCount_Text""))}";
            %end;
            put "          ));";
         %end;
         put "    );";
                  
      %end;
      put " VListBox(";   
		%if %length(&TEAEText) %then %do;
			put "objRefNS:TEAEText = TextBox(""     "" ||""&TEAEText"", <<Set Wrap(1000)),";
		%end;                           
		put "    HListBox(";
		%if %upcase(&CountMultiple)=NO %then %do;
			put "   objRefNS:CPTB = TextBox(""     "" || locale:fill(locale:get(""DisplayedCountsIndicateNumSubjExperiencingAnX_Text""),{""&textlabel""}), <<Set Wrap(650),<<Set Font Size(9)),";
		%end;
		%else %if %upcase(&CountMultiple)=YES %then %do;
			put "   objRefNS:CPTB = TextBox(""     "" || locale:fill(locale:get(""DisplayedCountsIndicateTotalNumOccOfAnX_Text""),{""&textlabel""}), <<Set Wrap(650),<<Set Font Size(9)),";
		%end;
		%if %upcase(&CountMultiple) = NO %then %do;
			put "       TextBox(""  ""),";
			put "       objRefNS:pcbb = ButtonBox(locale:get(""ShowPercents_Text""),<<UnderlineStyle(1),";
			put "       <<Set Function(Function({this},";
			put "          objRefNS:CurrBName = this << Get Button Name();";
			put "          If(objRefNS:CurrBName == locale:get(""ShowPercents_Text""),";
			put "             objRefNS:GBPexpr;";
			put "             objRefNS:Percents = 1;";
			put "             objRefNS:CPTB<< Set Text(""     "" || locale:fill(locale:get(""DispValsIndPercSubjWithinDemGrpExpAnX_Text""),{""&textlabel""}));"; 
			put "             this << setButtonName(locale:get(""ShowCounts_Text""));";
			put "             ,";
			%do i = 1 %to &ntv;
				put "          objRefNS:gbr&i = report(objRefNS:gb&i)[Graph Builder Box(1)];";
				put "          objRefNS:gb&i << SendToReport(";
				put "             //Dispatch({},""Graph Builder"",OutlineBox,{Set Title( locale:fill(locale:get(""XCountsGraph_Text""),{""%upcase(&domain)""}) )}),";          
				put "             Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),"; 
				put "             Dispatch({},""Y title"", TextEditBox,";
				put "                {Set Text("" "")}";
				put "          ));";
				put "          objRefNS:gbr&i << Remove Variable({:Name(""%  ""), Role(""Y"")});";
			%end; 
			put "             objRefNS:Percents = 0;";
			put "             objRefNS:GBCexpr;";
			put "             objRefNS:CPTB << Set Text(""     "" || locale:fill(locale:get(""DisplayedCountsIndicateNumSubjExperiencingAnX_Text""),{""&textlabel""}));";
			put "             this << setButtonName(locale:get(""ShowPercents_Text""))";
			put "       ))));";
		%end;
		put "    ),";
      put "      objRefNS:gbthlb = Vlistbox(";
      /* Draw distributions displays initially */
		%do i=1 %to &ntv;
			%if %symexist(dbv1) %then %do;
				%if &NtrtPeriods > 1 %then %do;
					%barplot1(&i,%quote(""&dbv1""),%quote(""Period""));
				%end;
				%else %do;
					%barplot1(&i,%quote(""&dbv1""),%str());
				%end;    
			%end;
			%else %do;
				%barplot1(&i,%str(),%str());
			%end;
		%end;
		%if &ntv %then %do;
			put " objRefNS:GBCexpr; ";
		%end;
      %else %do;
         put "      objRefNS:dd = Distribution(";
         put "            Arrange in Rows(3),";
         put "            Automatic Recalc( 1 ),";
         %if %upcase(&SplitResults)=YES %then %do;
            put "         By(eval(objRefNS:dbv)),";
         %end;
         %if %index(&countplotVars,&TmName)=0 %then %do;
            put "         Nominal Distribution(Column(:Name(""&DecodLabel"")), ShowCounts(1),Order By( ""Count Ascending"" )),";
         %end;
         %do i=1 %to &nev;
            put "         Nominal Distribution(Column(:Name(""&&evl&i"")), ShowCounts(1)),";
         %end;       
         %if%length(&StdyLabel) %then %do;
            put "         Continuous Distribution(Column(:Name(""&StdyLabel"")), ShowCounts(0), Quantiles(0),Summary Statistics(0)),";
         %end;          
         put "         SendToReport(";
         %if %length(&StdyLabel) %then %do;
            put " //            Dispatch( {""&StdyLabel""}, ""Distrib Histogram"", FrameBox, {Frame Size(85, 275)} ),";
         %end; 
         %if %index(&countplotVars,&TmName)=0 %then %do;
            put "            Dispatch( {""&DecodLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;
         %do i=1 %to &nev;
               put "            Dispatch( {""&&evl&i""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;

         put "         )";
         put "      ),";
      %end;
      put "     ),";
      put " );  //end listboxes"; 
      put ");";  /* End function for count plot content */
      
      /* Expression for Tabbed report to insert data filter into LHP */
         put "objRefNS:AddDataFilterContextBox = 1;";
         put "objRefNS:filterColumnsList = {};";
         %if %length(&DeltaLabel) %then %do;
            put "insertInto(objRefNS:filterColumnsList, {{""&DeltaLabel"", {}}});";
         %end;
         %if ^%index(%Upcase(&DesignByVars),STUDYID) and &StudyLevs > 1 %then %do;
            put "insertInto(objRefNS:filterColumnsList, {{""&StudyLabel"", {}}});";
         %end;         
         %if %upcase(&CountMultiple)=NO %then %do;
            put "insertInto(objRefNS:filterColumnsList, {{""&tclabel"", {}}});";
            put "insertInto(objRefNS:filterColumnsList, {{""&polabel"", {}}});";
         %end;
         %if &doTreatmentRRs %then %do;
            %if %eval(&nlev) > 2 %then %do;
               put "insertInto(objRefNS:filterColumnsList, {{""&maxrrlabel"", {}}});";
            %end;
            %else %do;
               %do i = 1 %to %eval(&nlev);
                  %if &i ^= &plev %then %do;
                     put "insertInto(objRefNS:filterColumnsList, {{""&&rrlabel&i"", {}}});";
               %end;
               %end;
            %end;
            put "insertInto(objRefNS:filterColumnsList, {{""&trtocclabel"", {}}});";
         %end;
         %if %length(&customFilterVars) %then %do;
			 %do i=1 %to &ndfv;
				put "insertInto(objRefNS:filterColumnsList, {{""&&dfvl&i"", {}}});";
			 %end;
	     %end;
	     %else %do;
			 %do i=1 %to &nev;
				put "insertInto(objRefNS:filterColumnsList, {{""&&evl&i"", {}}});";
			 %end;
	     %end;	     
      put "objRefNS:ReportDataFilterReferenceList = reportRefNS:createReportFilter(objRefNS:dmd, objRefNS:filterColumnsList);";
      
      %if &ntv %then %do; 
         put "objRefNS:AddLeftControlPanel = 1;";
         put "objRefNS:ddwa = Function({},{default local},";
         put "   objRefNS:LeftControlVBox << Append(";
         put "    VListBox(OutlineBox(locale:get(""View_Text""),";
         put "     Outlinebox(locale:get(""DemographicGrouping_Text""),";
         put "     objRefNS:glb=ListBox(objRefNS:tmlist,";
         %if %Upcase(&TrtUse) ne NONE %then %do;
            put "       maxselected(1),nlines(nitems(objRefNS:tmlist)),<<setselected(2,1)";
         %end;
         %else %do;
            put "       maxselected(1),nlines(nitems(objRefNS:tmlist)),<<setselected(1,1)";
         %end;
         put "   )),";
         %if ^%symexist(DOMAINLABEL_&domain) %then %let DOMAINLABEL_&domain = %nrbquote(&domain);
         put "   OutlineBox(locale:fill(locale:get(""XStacking_Text""), {""&&DOMAINLABEL_&domain""}),";
         put "    objRefNS:glb2 = ListBox(objRefNS:tm2list,";
         %if &NTrtPeriods > 1 %then %do;
            put "       maxselected(1),nlines(nitems(objRefNS:tm2list)),<<setselected(2,1)";
         %end;
         %else %do;
            put "       maxselected(1),nlines(nitems(objRefNS:tm2list)),<<setselected(1,1)";
         %end;
         put "   )))";
         put "  ));";      
         /* kemicl - script for column switcher box */            
         put " objRefNS:glb << Set Script(";
         put "    SelectOK = 1;";
         put "    try((objRefNS:glb<<GetSelected)[1],SelectOK = 0);";
         put "    If(SelectOK,";
         put "   DtExpr;";
         put "   SVar = (objRefNS:glb<<GetSelected)[1];";
         put "   SVar2 = (objRefNS:glb2<<GetSelected)[1];";
         put "   SPVar = objRefNS:CPArray[Svar];";
         put "   SNVar = objRefNS:CNArray[Svar];";
            /* Kemicl - update the formula for percent column to use right denominator */ 
         put "    Eval(parse(EvalInsert(""\[Column(""%  "")<<Set Formula(:Name(""^SPvar^""))]\"")));";
         put "    Eval(parse(EvalInsert(""\[Column(locale:get(""N_Text""))<<Set Formula(:Name(""^SNvar^""))]\"")));";
         
         %do i=1 %to &ntv;
         	put "   objRefNS:gbr&i = report(objRefNS:gb&i)[Graph Builder Box(1)];";
         	put "	  objRefNS:gbVars = objRefNS:gbr&i<<GetVariables();";
         	put "	  objRefNS:groupVarIndex = objRefNS:findVarByRole(objRefNS:gbVars,""Group Y"");";
         	put "	  if(objRefNS:groupVarIndex, objRefNS:gbr&i<<RemoveVariable(objRefNS:groupVarIndex));";
				put "   objRefNS:tbhlb&i << Delete;";
            put "   dbv2 = {};";
            put "   dbvFirst = {};";
            put "   If(SVar == ""&trNone"",";
               put "    If(SVar2 != ""&trNone"",";
               put "       InsertInto(dbv2,Column((objRefNS:glb2<<GetSelected)[1]));";
               put "    );";     
            put " ,";  /*else for grouping var */
            	put "	   objRefNS:gbr&i << AddVariable({Column(Svar),Role(""Group Y"")});";
            	put "    InsertInto(dbv2,Column((objRefNS:glb<<GetSelected)[1]));";
            	put "    InsertInto(dbvFirst,Column((objRefNS:glb<<GetSelected)[1]));";            
            	put "    If(Svar2 != ""&trNone"",";
               put "    	InsertInto(dbv2,Column((objRefNS:glb2<<GetSelected)[1]));";
            	put "    );";
            put "  );";          
            put "    objRefNS:tbvlb << Append( ";
                        %tabulate1(&i,%nrstr(eval(dbv2)),%nrstr(eval(dbvFirst)));
            put "    );";
         %end;
         /* else if nothing selected, set to NONE */
         put "    ,";
         put "    objRefNS:glb << SetSelected(1,1);";
         put "    );";  /*end SelectOK condition */
         put " );";  /* End Column Switcher script */ 
         
         /* Script for stacking columns */
         /* kemicl - script for column switcher box */            
         put " objRefNS:glb2 << Set Script(";
         put "    SelectOK = 1;";
         put "    try((objRefNS:glb2<<GetSelected)[1],SelectOK = 0);";
         put "    If(SelectOK,";
         put "   DtExpr;";
         %do i=1 %to &ntv;
            put "   objRefNS:gbr&i = report(objRefNS:gb&i)[Graph Builder Box(1)];";
         	put "	  objRefNS:gbVars = objRefNS:gbr&i<<GetVariables();";
         	put "	  objRefNS:groupVarIndex = objRefNS:findVarByRole(objRefNS:gbVars,""Overlay"");";
         	put "	  if(objRefNS:groupVarIndex, objRefNS:gbr&i<<RemoveVariable(objRefNS:groupVarIndex));";
         	put "	  objRefNS:tbhlb&i << Delete;";
            put "   SVar2 = (objRefNS:glb2<<GetSelected)[1];";
            put "   SVar = (objRefNS:glb<<GetSelected)[1];";
            put "   dbv2 = {};";
            put "   dbvFirst = {};";
            put "   If(SVar2 == ""&trNone"",";
            put "    If(SVar != ""&trNone"",";
               put "    InsertInto(dbv2,Column((objRefNS:glb<<GetSelected)[1]));";
               put "    InsertInto(dbvFirst,Column((objRefNS:glb<<GetSelected)[1]));";               
            put "    );";
            put "    objRefNS:tbvlb << Append( ";
                        %tabulate1(&i,%nrstr(eval(dbv2)),%nrstr(eval(dbvFirst)));
            put "    );";        
            put " ,";  /* else for stacking var */
				put "	   objRefNS:gbr&i << AddVariable({Column(Svar2),Role(""Overlay"")});";            
            put "    InsertInto(dbv2,Column((objRefNS:glb2<<GetSelected)[1]));";
            put "    If(SVar != ""&trNone"",";
               put "    InsertInto(dbv2,Column((objRefNS:glb<<GetSelected)[1]),1);";
               put "    InsertInto(dbvFirst,Column((objRefNS:glb<<GetSelected)[1]));";      
            put "    );";
            put "    objRefNS:tbvlb << Append( ";
                        %tabulate1(&i,%nrstr(eval(dbv2)),%nrstr(eval(dbvFirst)));
            put "    );";           
            put "  );";
         %end;
         /* else if nothing selected, set to NONE */
         put "    ,";
         put "    objRefNS:glb2 << SetSelected(1,1);";         
         put "    );";  /*end SelectOK condition */
         put " );";  /* End stacking Switcher script */           
         put ");";   /* End column switcher objRefNS:dd Function */  
      %end; 
   
      %if &ntv %then %do;
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""CountsGraph_Text"")));";
         put "InsertInto(objRefNS:CreateReportList, {{objRefNS:TEAEText, objRefNS:gb1";
         %do i = 2 %to &ntv;
            put ",objRefNS:gb&i";
         %end;
         put "}});";
      %end;
      %else %do;
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""Distributions_Text"")));";
         put "InsertInto(objRefNS:CreateReportList, {{objRefNS:dd}});";
      %end;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
      put "InsertInto(objRefNS:DataNameList,{{locale:fill(locale:get(""XDistribution_Text""), {""%upcase(&domain)""})}});";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ddw()));";
      %if (&ntv) %then %do;
         put "InsertInto(objRefNS:TabAfterCodeList, expr(objRefNS:ddwa()));";
         put "objRefNS:CloseTabOB = 1;";
      %end;
      %else %do;
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";     
      %end;
      put "InsertInto(objRefNS:TabOpenList,1);";
       
		put " objRefNS:tbfunc = Function({}, {default local},";
		put "    DtExpr;";
		put "    objRefNS:tbvlb = Vlistbox(";
		%if %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
			put "    objRefNS:tvcol = {Column(""&TrtLabel"")};";
			put "    objRefNS:tvfcol = objRefNS:tvcol;";
			%if &NtrtPeriods > 1 %then %do;
				put "    InsertInto(objRefNS:tvcol,{Column(""Period"")});";
			%end;
			%do i=1 %to &ntv;
				%tabulate1(&i,%nrstr(eval(objRefNS:tvcol)),%nrstr(eval(objRefNS:tvfcol))); 
			%end;
		%end;
		%else %do;
			%do i=1 %to &ntv;
				%tabulate1(&i,%nrstr({}),%nrstr({})); 
			%end;
		%end;
		put "    );";
		put " );";

      %if &ntv  %then %do;
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""CountsTable_Text"")));";
         put "InsertInto(objRefNS:CreateReportList, {{objRefNS:tbhlb1";
         %do i = 2 %to &ntv;
            put ",objRefNS:tbhlb&i";
         %end;
         put "}});";
         put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
         put "InsertInto(objRefNS:DataNameList,{{locale:fill(locale:get(""XDistribution_Text""), {""%upcase(&domain)""})}});";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:tbfunc()));";
         put "InsertInto(objRefNS:TabAfterCodeList,"" "");";
         put "objRefNS:CloseTabOB = 1;";
         put "InsertInto(objRefNS:TabOpenList,1);";         

         put "objRefNS:ddwd=Function({},{default local},";
         put "DtExpr;";
         put "objRefNS:dbvd = {};";
         %do i=1 %to &ndbv;
            %if %quote(&&dbv&i) ne %quote(&TrtLabel) %then %do;
               put "InsertInto(objRefNS:dbvd,Column(""&&dbv&i""));";
            %end;
         %end;
         put " VListBox(";   
         put "      objRefNS:dd2 = Distribution(";
         put "               //Arrange in Rows(4),";
         put "            Automatic Recalc( 1 ),";
         %if %upcase(&SplitResults)=YES %then %do;
            put "         By(eval(objRefNS:dbvd)),";
         %end;

         put "         Nominal Distribution(Column(:Name(""&DecodLabel"")),ShowCounts(1), Order By( ""Count Ascending"" )),";
         %if %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
            put "          Nominal Distribution(Column(:Name(""&TrtLabel"")), ShowCounts(1)),";
            %if &NtrtPeriods > 1 %then %do;
               put "          Nominal Distribution(Column(:Name(""Period"")), ShowCounts(1)),"; 
            %end;
         %end;
        %if &SLDOSEVar %then %do;
               put "         Continuous Distribution(Column(:Name(""&DoseLabel"")), ShowCounts(0), Quantiles(0),Summary Statistics(0)),";
         %end;             
         %if %length(&StdyLabel) %then %do;
            put "         Continuous Distribution(Column(:Name(""&StdyLabel"")), ShowCounts(0), Quantiles(0),Summary Statistics(0)),";
         %end;              
         %do i=1 %to &nev;
            put "         Nominal Distribution(Column(:Name(""&&evl&i"")), ShowCounts(1)";
            %if %length(&GroupVarLabel)  %then %do;
               %if "&&evl&i" = "&GroupVarLabel" %then %do;
                  put "       ,Order By( ""Count Ascending"" )";
               %end;
            %end;
            put "),";
         %end; 
         put "         SendToReport(";
         %if %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
            put "            Dispatch( {""&TrtLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;
         put " //          Dispatch( {""&StdyLabel""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
         put " //          Dispatch( {""&StdyLabel""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";  
         put "            Dispatch( {""&DecodLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %do i=1 %to &nev;
               put "            Dispatch( {""&&evl&i""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;

         put "         )";
         put "      ))";
         put " );";
         put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
         put "InsertInto(objRefNS:DataNameList,{{locale:fill(locale:get(""XDistribution_Text""), {""%upcase(&domain)""})}});";
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""Distributions_Text"") || "" ""));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ddwd()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:TabOpenList,1);";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:dd2}});";
      %end;

      %SubjectDrillDowns;

		put "InsertInto(objRefNS:ActionButtonDesc , {Expr(locale:get(""ClicktoView_Text""))});";;
		put "Include(""$CLINICAL_HOME\JSLFiles\Distribution Drill Down Functions.jsl"");";   

		%if &doSite %then %do;
			put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""DemographicCounts_Text""))});";
			put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:SSfctn(""&SiteLabel"",""&TrtLabel"", ""&AxLab"")));";
			put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.DemographicCounts.gif"");";
			put "InsertInto(objRefNS:ActionSelectionCountList[1],1);";           
			put "InsertInto(objRefNS:ActionRequirementFlagList[1],0);";
		%end;    
		/*
		%if %upcase(&domain) = CM or %upcase(&domain) = AE %then %do;
			%if %sysfunc(findw(&domains, %upcase(&crossdomain))) > 0 %then %do;
				put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:fill(locale:get(""RelatedX_Text""),{""%upcase(&crossdomain)""}))});";
				put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:AECMfctn(""&crossdomain"",""&APddname"",locale:fill(locale:get(""RelatedX_Text""),{""%upcase(&crossdomain)""}))));";
				%if %upcase(&crossdomain) = AE %then %let RXIcon = %nrbquote(&IconPath.RelatedAE.gif);
				%else %if %upcase(&crossdomain) = CM %then %let RXIcon = %nrbquote(&IconPath.RelatedConcomitantMedications.gif);
				%else %if %upcase(&crossdomain) = EG %then %let RXIcon = %nrbquote(&IconPath.RelatedECG.gif);
				%else %if %upcase(&crossdomain) = LB %then %let RXIcon = %nrbquote(&IconPath.Labs.gif);
				%else %if %upcase(&crossdomain) = VS %then %let RXIcon = %nrbquote(&IconPath.RelatedVitals.gif);
				%else %let RXIcon = %nrbquote(&IconPath.DonutGraph3d.gif);
				put "InsertInto(objRefNS:ActionIconList[2],""&RXIcon"");";
				put "InsertInto(objRefNS:ActionSelectionCountList[2],1);";
				put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
			%end;
		%end;                
		%if %sysfunc(findw(&domains, LB)) %then %do;
			put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:get(""RelatedLabs_Text""))});";
			put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:AECMfctn(""LB"",""FindingsTimeTrends"",locale:get(""RelatedLabs_Text""))));";
			put "InsertInto(objRefNS:ActionIconList[2],""&IconPath.Labs.gif"");";
			put "InsertInto(objRefNS:ActionSelectionCountList[2],1);";           
			put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
		%end;
		%if %sysfunc(findw(&domains, VS))>0 %then %do;
			put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:get(""RelatedVitals_Text""))});";
			put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:AECMfctn(""VS"",""FindingsTimeTrends"",locale:get(""RelatedVitals_Text""))));";
			put "InsertInto(objRefNS:ActionIconList[2],""&IconPath.RelatedVitals.gif"");";
			put "InsertInto(objRefNS:ActionSelectionCountList[2],1);";           
			put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
		%end;
		%if %sysfunc(findw(&domains, EG))>0 %then %do;
			put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:get(""RelatedECG_Text""))});";
			put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:AECMfctn(""EG"",""FindingsTimeTrends"",locale:get(""RelatedECG_Text""))));"; 
			put "InsertInto(objRefNS:ActionIconList[2],""&IconPath.RelatedECG.gif"");";
			put "InsertInto(objRefNS:ActionSelectionCountList[2],1);";           
			put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
		%end; 
		*/
		%if %upcase(&domain) = AE %then %do;
			put "objRefNS:ActionButtonList[2]={};";
			put "objRefNS:ActionCodeList[2] ={};";
			put "objRefNS:ActionIconList[2] = {};";
			put "objRefNS:ActionSelectionCountList[2] = {};";
			put "objRefNS:ActionRequirementFlagList[2] = {};";		
			put "objRefNS:ActionButtonList[2]={expr(locale:get(""UniqueOccurrenceSubjectCounts_Text""))};";
			put "objRefNS:ActionCodeList[2] ={expr(objRefNS:AECountsfctn(""&SubjectLabel"", ""&DecodLabel""))};";
			put "objRefNS:ActionIconList[2] = {""&IconPath.SingleOccuranceSubjectCountReport.gif""};";
			put "objRefNS:ActionSelectionCountList[2] = {0};";
			put "objRefNS:ActionRequirementFlagList[2] = {0};";
		%end;

      
      
   run;

   %PackageInsertFile(jsl code from &ProcessName, &OutPath, &DistributionScriptName, &Distribution_OutDataName..sas7bdat, text/plain); 

   %mexit:;

%mend;

